/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis Søgaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */

package dk.firma.klient.webservice;

import java.io.FileInputStream;
import java.security.KeyStore;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import com.sun.deploy.security.RootCertStore;
import dk.gov.oiosi.communication.*;
import dk.gov.oiosi.security.RootCertificateConfig;
import dk.gov.oiosi.security.lookup.CertificateLoader;
import org.apache.axiom.om.OMElement;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.addressing.Identifier;
import dk.gov.oiosi.common.IdentifierUtility;
import dk.gov.oiosi.common.Utilities;
import dk.gov.oiosi.communication.client.TransportController;
import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.communication.configuration.ProfileMappingCollectionConfig;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.raspprofile.communication.RaspRequest;
import dk.gov.oiosi.security.CertificateSubject;
import dk.gov.oiosi.security.SignatureValidationProof;
import dk.gov.oiosi.security.ldap.LdapLookupFactory;
import dk.gov.oiosi.security.lookup.ICertificateLookup;
import dk.gov.oiosi.security.oces.OcesX509Certificate;
import dk.gov.oiosi.security.revocation.IRevocationLookup;
import dk.gov.oiosi.security.revocation.RevocationLookupFactory;
import dk.gov.oiosi.security.revocation.RevocationResponse;
import dk.gov.oiosi.uddi.EndpointAddressTypeCode;
import dk.gov.oiosi.uddi.IUddiLookupClient;
import dk.gov.oiosi.uddi.LookupParameters;
import dk.gov.oiosi.uddi.RegistryLookupClientFactory;
import dk.gov.oiosi.uddi.UddiGuidId;
import dk.gov.oiosi.uddi.UddiId;
import dk.gov.oiosi.uddi.UddiLookupResponse;
import dk.gov.oiosi.uddi.UddiStringId;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.XmlSchema;
import dk.gov.oiosi.xml.documenttype.DocumentTypeConfigSearcher;
import dk.gov.oiosi.xml.schema.SchemaValidator;
import dk.gov.oiosi.xml.schematron.SchematronValidator;
import sun.reflect.generics.reflectiveObjects.NotImplementedException;

/**
 * Invoke RASP call against ITST published RASP test service.
 * 
 */
public class OiosiRaspClient
{

    private static Log log = LogFactory.getLog(OiosiRaspClient.class);
    public static final String PATH_INVOICE_XML = "./resources/xml/OIOXML_invoice.xml";

    public enum UddiType {
        Production,
        Test,
        LocalJave_v1_2_1,
        LocalJave_v1_2_3,
        LocalNet_v1_2_1,
        LocalNet_v1_2_3
    }


    public UddiType uddiType;

    public String xmlDocumentUrl;

    public OiosiRaspClient()
    {
        this.uddiType = UddiType.Test;
        this.xmlDocumentUrl = PATH_INVOICE_XML;
    }

    public OiosiRaspClient(UddiType uddiType, String xmlDocumentUrl)
    {
        this.uddiType = uddiType;
        this.xmlDocumentUrl = xmlDocumentUrl;
    }



    /**
     * RASP Test application
     *
     * This test client goes through the following steps:
     *
     *  1. Load a document
     *  2. Validate the document
     *  3. Make an UDDI lookup to find the service address
     *  4. Use information from the UDDI response to download the server certificate via LDAP
     *  5. Check whether the downloaded certificate is revoked
     *  6. Send it to a service deployed on localhost (by default: the service_skeleton service)
     *
     */

    public static void main(final String[] args)
    {
        new OiosiRaspClient().sendDocument();
    }

    public boolean sendDocument()
    {
        boolean result = false;

        try {

            if(uddiType == UddiType.Production )
            {
                log.info("Sending the document using the production UDDI");
                ConfigurationHandler.setConfigurationFile("cfg/RaspConfiguration.Production.xml");

            }
            else if (uddiType == UddiType.Test)
            {
                log.info("Sending the document using the test UDDI");
                ConfigurationHandler.setConfigurationFile("cfg/RaspConfiguration.Test.xml");
            }
            else if(uddiType == UddiType.LocalJave_v1_2_1)
            {
                log.info("Sending the document to the local Java 1.2.1 service");
                ConfigurationHandler.setConfigurationFile("cfg/RaspConfiguration.Test.xml");
            }
            else if(uddiType == UddiType.LocalJave_v1_2_3)
            {
                log.info("Sending the document to the local Java 1.2.3 service");
                ConfigurationHandler.setConfigurationFile("cfg/RaspConfiguration.Test.xml");
            }
            else if (uddiType == UddiType.LocalNet_v1_2_1)
            {
                log.info("Sending the document to the local .Net rasp service");
                ConfigurationHandler.setConfigurationFile("cfg/RaspConfiguration.Test.xml");
            }
            else if(uddiType == UddiType.LocalNet_v1_2_3)
            {
                log.info("Sending the document to the local Java service");
                ConfigurationHandler.setConfigurationFile("cfg/RaspConfiguration.Test.xml");
            }
            else
            {
                throw new Exception("Uddi type not recognized!!");
            }

            log.info("Try to send message to service");
            
            // Set the path to the custom RASP configuration file. 
            // This file contains setup for things like:
            //   - Addresses to OCSP, LDAP and UDDI servers
            //   - The path to the Axis2 modules needed (RASP, WS-Security and WS-ReliableMessaging)
            //   - Configuration for the different OIO UBL document types
            // ConfigurationHandler.setConfigurationFile("cfg/RaspConfiguration.xml");

            
            // Step 1: Load a document
            
            // Load the document from disc
            XmlDocument xmlDocument = new XmlDocument();
            xmlDocument.load(xmlDocumentUrl);
            log.info("Document created from file - " + xmlDocumentUrl);

            // Search our configuration file for the document type we are currently about to send.
            // The UBL type can be inferred from the XML root element name of the file, and the configuration we get
            // contains paths to XML schemas, schematron stylesheets and XPath expressions used to find the EAN number for example
            DocumentTypeConfigSearcher documentTypeSearcher = new DocumentTypeConfigSearcher();
            DocumentTypeConfig documentTypeConfig = documentTypeSearcher.findUniqueDocumentType(xmlDocument);
            log.info("Document is known in configuration");

            
            // Step 2: Validate the document
            
            // Schema validate document before sending...
            SchemaValidator schemaValidator = new SchemaValidator();
            XmlSchema schema = new XmlSchema(documentTypeConfig.getSchemaPath());
            schemaValidator.schemaValidateXmlDocument(xmlDocument, schema);
            log.info("Document is schema valid");

            // Schematron validate Document before sending...
            SchematronValidator schematronValidator = new SchematronValidator(documentTypeConfig.getSchematronValidationConfig());
            schematronValidator.schematronValidateXmlDocument(xmlDocument);
            log.info("Document is schematron/logically valid");

            Uri endpointAddress = null;

            OcesX509Certificate ocesCertificateClient = null;
            OcesX509Certificate ocesCertificateServer = null;

            // Step 3: Make UDDI Lookup

            // Info about the service url (endpointAddress)
            // If the url ends with '/nemhandel' as in http://localhost:8080/Rasp-1.2.3/services/TestService/nemhandel
            // the endpoint accept all kinds of documents.
            // If the endpoint does not end with '/nemhandel' only the documents defined in the file
            // ~/dk.gov.oiosi.examples.service\WebContent\WEB-INF\services\TestService\META-INF\service.xml
            // in an element serviceGroup/service/operation
            // can be received.
            // At pressent only Invoice (OIOXML and OIOUBL2) is defined.
            //
            // The url for the service can be changed after deployment. The default is ~/services/TestService/nemhandel, but the TestService
            // can be changed to whatever suit you business.
            //  1. Deploy the war file.
            //  2. Manual rename the folder 'TestService' to the desired name (keep in mind, that the url should end on '/nemhandel'
            //     as described above.
            //  3. Manualy change the services.xml file located in
            //    ~/Tomcat 6.0\webapps\Rasp-1.2.3\WEB-INF\services\TestService\META-INF/services.xml
            //     Change the attribute serviceGroup/service@name='???' to the desired name

            if(uddiType == UddiType.Production || uddiType == UddiType.Test)
            {
                log.info("Sending the document though uddi lookup.");

                Identifier identifier = Utilities.getReceiverEndpointKey(xmlDocument);

                // Find profile definition id (based on profile name in xml document)
                ProfileMappingCollectionConfig profileMappingConfig = ConfigurationHandler.getInstance().getProfileMappingCollectionConfig();

                List<EndpointAddressTypeCode> endpointTypes = new LinkedList<EndpointAddressTypeCode>();
                endpointTypes.add(EndpointAddressTypeCode.http);

                LookupParameters parameters;
                if (documentTypeConfig.getProfileIdXPath().isEmpty()) {
                    parameters = new LookupParameters(
                            identifier,
                            IdentifierUtility.getUddiIDFromString(documentTypeConfig.getServiceContractTModel()),
                            endpointTypes
                            );
                } else {
                    UddiId profileUddiId = Utilities.getProfileId(xmlDocument, documentTypeConfig, profileMappingConfig);
                    List<UddiId> profileDefinitionIds = new ArrayList<UddiId>();
                    profileDefinitionIds.add(profileUddiId);


                    // Setup the UDDI lookup parameters
                    parameters = new LookupParameters(
                            identifier,
                            IdentifierUtility.getUddiIDFromString(documentTypeConfig.getServiceContractTModel()),
                            profileDefinitionIds,
                            endpointTypes
                            );
                }

                // Look it up...
                log.info("Ask UDDI for: " + identifier);
                IUddiLookupClient uddiClient = RegistryLookupClientFactory.getRegistryLookupClient();
                List<UddiLookupResponse> uddiResult = uddiClient.lookup(parameters);
                log.info("Found # in uddi : " + uddiResult.size());
                if (uddiResult.size() == 0) {
                    log.error("No Information found in UDDI");
                    result = false;
                    return result;
                }

                UddiLookupResponse uddiResponse = (UddiLookupResponse) uddiResult.get(0);
                log.info("Found endpoint         : " + uddiResponse.getEndpointAddress().getKeyAsString());
                log.info("Found Certificate Info : " + uddiResponse.getCertificateSubjectSerialNumber().getSerialNumberValue());

                // Step 4: LDAP Lookup
                // Get the certificate identifier from the UDDI results
                CertificateSubject certificateSubject = uddiResponse.getCertificateSubjectSerialNumber();

                // Use a factory to create an LDAP client, and download the certificate
                ICertificateLookup lookup = LdapLookupFactory.createLdapLookupClient();
                X509Certificate server = lookup.getCertificate(certificateSubject);
                log.info("Found Certificate in LDAP : " + server.getSubjectDN());


                // Step 5: Revocation Check

                // Use a factory to create a revocation client, and check the certificate status
                IRevocationLookup revocationLookup = RevocationLookupFactory.createRevocationLookupClient();
                RevocationResponse revocationResponse = revocationLookup.checkCertificate(server);
                log.info("Certificate is valid : " + revocationResponse.isValid());

                endpointAddress = uddiResponse.getEndpointAddress().getAsUri();

                // Use the downloaded certificate as the server side credential
                ocesCertificateServer = new OcesX509Certificate(server);

            }
            else if (uddiType == UddiType.LocalJave_v1_2_1)
            {
                log.info("Sending the document to the local Java 1.2.1 service.");
                endpointAddress = new Uri("http://localhost:8080/Rasp-1.2.1/services/TestService/nemhandel");

                // Load the server certificate from keystore
                KeyStore ks = KeyStore.getInstance("JKS");
                ks.load(new FileInputStream("./keys/OCES1_Test.jks"), "Test1234".toCharArray());
                X509Certificate server = (X509Certificate) ks.getCertificate("function");
                ocesCertificateServer = new OcesX509Certificate(server);
            }
            else if (uddiType == UddiType.LocalJave_v1_2_3)
            {
                log.info("Sending the document to the local Java 1.2.3 service.");

                // if endpoint end withs /nemhandel, the service accept all documents
                // else it only accept the document that is linked in the services.xml file
                endpointAddress = new Uri("http://localhost:8080/Rasp-1.2.3/services/TestService/nemhandel");

                // Load the server certificate from keystore
                KeyStore ks = KeyStore.getInstance("JKS");
                ks.load(new FileInputStream("./keys/OCES1_Test.jks"), "Test1234".toCharArray());
                X509Certificate server = (X509Certificate) ks.getCertificate("function");
                ocesCertificateServer = new OcesX509Certificate(server);

                // http://testservice.nemhandel.gov.dk:8080/integration/RASPJava/1.2.3.Hotfix1/services/receiver
            }
             else if (uddiType == UddiType.LocalNet_v1_2_1)
            {
                log.info("Sending the document to the local .Net 1.2.1 service.");
                result = false;
                return result;


            }else if (uddiType == UddiType.LocalNet_v1_2_3)
            {
                log.info("Sending the document to the local Net 1.2.3 service.");
                result = false;
                return result;
            }
            else
            {
                 throw new Exception("The uddi type '"+uddiType.toString()+"' is unknown, not supported.");
            }

            // Endpoint identified
            log.info("Endpoint identified    : "+endpointAddress);

            if(endpointAddress.getAbsoluteUri().contains("/nemhandel"))
            {
                log.info("Destination service should accept all document types.");
            }
            else
            {
                log.info("Destination service only accept the document types defined in the ~/META-INF/service.xml.");
                // ~/dk.gov.oiosi.examples.service\WebContent\WEB-INF\services\TestService\META-INF\service.xml
            }

            /*
             * Load own certificate. This is used for retrieving sender id, when a confidential sender id is present
             * in the document to be sent. Note that this certificate is not the one used for signing and encryption.
             *
             * The certificate used for signing and encrypting is specified in the file policy.xml.
             */
            // Load the client certificate from keystore
            X509Certificate client;

            if(uddiType == UddiType.Production)
            {
                // load the certificate from windows key store
                //RootCertificateConfig config = new RootCertificateConfig();
                RootCertificateConfig config = ConfigurationHandler.getInstance().getRootCertificateConfig();

                //config.
                client = CertificateLoader.GetCertificateFromCertificateStoreInformation(config);
            }
            else if(uddiType == UddiType.Test
                    || uddiType == UddiType.LocalJave_v1_2_1
                    || uddiType == UddiType.LocalJave_v1_2_3
                    || uddiType == UddiType.LocalNet_v1_2_1
                    || uddiType == UddiType.LocalNet_v1_2_3)
            {
                KeyStore ks2 = KeyStore.getInstance("JKS");
                ks2.load(new FileInputStream("./keys/OCES1_Test.jks"), "Test1234".toCharArray());
                client = (X509Certificate) ks2.getCertificate("employee");
            }
            else
            {
                 throw new Exception("The uddi type '"+uddiType.toString()+"' is unknown, not supported.");
            }

            ocesCertificateClient = new OcesX509Certificate(client);




            log.info("Destination service certificate : " + ocesCertificateServer.getSubject().getSerialNumberValue());

            //Step 6: Send it
            // Create the credentials
            Credentials credentials = new Credentials(ocesCertificateClient, ocesCertificateServer);
            
            // Use policy defined in RaspConfiguration.xml
            SendPolicy sendPolicy = ConfigurationHandler.getInstance().getSendPolicy();
            log.info("Sendpolicy loaded from configuration");

            // Create a request
            RaspRequest request = new RaspRequest(new Request(endpointAddress, credentials, sendPolicy));

            // Send request to service
            Response response = request.getResponse(new OiosiMessage(xmlDocument), "urn:uuid:" + System.nanoTime());
            
            OMElement body = response.getResponseMessage().getMessageXml().getOMDocument();
            
            SignatureValidationProof signatureProof = response.getResponseMessage().getSignatureProof();
            
            log.info("Received response from service: " + (body == null ? "empty message" : body));
            log.info("SignatureValidationProof: " + signatureProof);

            // All okay, the document has been send and recieved okay
            result = true;
        }
        catch (Exception e)
        {
            log.error("Failed to send message to service : " + e, e);
        }

        try {
            // Let the ReliableMessaging session finish...
            long sleep = 60000;
            log.info("Now sleep for "+sleep +" milisec - Let the ReliableMessaging session finish...");
        	Thread.sleep(sleep);
            TransportController.shutdown();
            log.info("ReliableMessaging session finish and transportController.shutdown.");
        }
        catch (Exception ignore)
        {
        }

        log.info("Done...");

        return result;
    }
}