/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis Søgaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */

package dk.gov.oiosi.axis2.module.signatureproof;

import java.security.cert.X509Certificate;

import org.apache.axis2.AxisFault;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.description.HandlerDescription;
import org.apache.axis2.description.Parameter;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.rampart.handler.WSSHandlerConstants;
import org.apache.ws.security.handler.WSHandlerConstants;

import dk.gov.oiosi.Constants;
import dk.gov.oiosi.axis2.module.RaspHandler;
import dk.gov.oiosi.axis2.module.RaspModule;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.policy.OiosiPolicyBean;
import dk.gov.oiosi.security.SignatureValidationProof;
import dk.gov.oiosi.security.SignatureValidationProofAllreadyCompletedException;
import dk.gov.oiosi.security.revocation.IRevocationLookup;
import dk.gov.oiosi.security.revocation.RevocationLookupFactory;
import dk.gov.oiosi.security.revocation.RevocationResponse;
import dk.gov.oiosi.security.revocation.RevocationException;
import dk.gov.oiosi.security.validation.CertificateExpiredException;
import dk.gov.oiosi.security.validation.CertificateNotActiveException;
import dk.gov.oiosi.security.validation.CertificateValidator;
import dk.gov.oiosi.util.AxisWSSUtil;

/**
 * Create signature proof in axis stack.
 */
public class SignatureProofHandler extends RaspHandler {

    private static final Log log = LogFactory.getLog(SignatureProofHandler.class);

    /**
     * Name of the handler
     */
    public static final String NAME = "SignatureProofHandler";

    private static HandlerDescription EMPTY_HANDLER_METADATA = new HandlerDescription("deafult Handler");
    private HandlerDescription handlerDesc;

    /**
     * Constructor.
     */
    public SignatureProofHandler() {
        this.handlerDesc = EMPTY_HANDLER_METADATA;
    }

    @Override
    public void init(HandlerDescription handlerdesc) {
        this.handlerDesc = handlerdesc;
    }

    @Override
    public InvocationResponse invoke(MessageContext msgContext) throws AxisFault {

        log.debug("SignatureProofHandler : invoke");

        OiosiPolicyBean oiosiPolicyBean = getPolicyBean(msgContext);
        if (!oiosiPolicyBean.doSignatureProof()) {
            log.warn("SignatureProofHandler not activated. Check your policy configuration.");
            return InvocationResponse.CONTINUE;
        }
        
        if (!msgContext.isEngaged(RaspModule.MODULE_NAME)) {
        	log.error("SignatureProofHandler is missing prerequisite: Rasp module not engaged");
        	return InvocationResponse.ABORT;
        }
        
        if (!msgContext.isEngaged(WSSHandlerConstants.SECURITY_MODULE_NAME)) {
        	log.error("SignatureProofHandler is missing prerequisite: Security module not engaged");
            return InvocationResponse.ABORT;
        }

        if (RaspModule.getInstance().isRMNamespaceByAction(msgContext.getSoapAction())) {
            // The message is part of the RM handshake, so do not create signatureproof
            return InvocationResponse.CONTINUE;
        }

        Object recv_results = msgContext.getProperty(WSHandlerConstants.RECV_RESULTS);
        if (recv_results == null) {
        	log.error("The received result was null");
        	return InvocationResponse.ABORT;
        }
        
        X509Certificate signerCertificate = AxisWSSUtil.getCertificateFrom_RECV_RESULTS(recv_results);
        if (signerCertificate == null) {
        	log.error("Could not read signer certificate");
        	return InvocationResponse.ABORT;
        }
    	msgContext.setProperty(Constants.RASP_SIGNING_CERTIFICATE, signerCertificate);
    	
    	// Check if the certificate is valid.
    	boolean passedRevocation = false;
    	boolean passedValidation = false;
    	
        // Check the validity of the certificate via an OCSP lookup
        log.debug("Check Certificate : " + signerCertificate.getSubjectX500Principal());
        RevocationResponse response = null;
        try {
            IRevocationLookup lookup = RevocationLookupFactory.createRevocationLookupClient();
            response = lookup.checkCertificate(signerCertificate);

            if (response.isValid()) {
            	passedRevocation = true;
            } else {
            	log.warn("Certificate is not valid.");
            }
        } catch (RevocationException e) {
            log.warn("Validation of certificate failed " + signerCertificate.getSubjectX500Principal() + " : " + e);
        } catch (MainException e) {
            log.warn("Validation of certificate failed " + signerCertificate.getSubjectX500Principal() + " : " + e, e);
        } catch (Exception e) {
            log.warn("Could not validate incoming signer certificate : " + e, e);
        }

        try {
            CertificateValidator.validateCertificate(signerCertificate);
            passedValidation = true;
        } catch (CertificateNotActiveException e) {
            log.warn("Incoming certificate not valid yet : " + signerCertificate.getSubjectX500Principal());
        } catch (CertificateExpiredException e) {
            log.warn("Incoming certificate is expired : " + signerCertificate.getSubjectX500Principal());
        }
        
        // Calculates whether the certificate is valid
        boolean certificateValid = passedRevocation && passedValidation;
        
        try {
            SignatureValidationProof signatureProof = new SignatureValidationProof();
            signatureProof.setCertificateSubject(signerCertificate.getSubjectX500Principal().toString());
            signatureProof.setValidCertificate(certificateValid);

            // Rampart do not let invalid signatures through
            signatureProof.setValidSignature(true);
            signatureProof.setUnchangedMessage(true);

            // Check if message was encrypted
            signatureProof.setEncryptedMessage(AxisWSSUtil.getMessageEncryptedFrom_RECV_RESULTS(recv_results));

            // Get timestamp from SecurityHeader 
            signatureProof.setTimeStamp(AxisWSSUtil.getTimestampCreatedFrom_RECV_RESULTS(recv_results));

            // set completed
            signatureProof.setCompleted();

            msgContext.setProperty(Constants.RASP_SIGNATURE_PROOF, signatureProof);
                        
        } catch (SignatureValidationProofAllreadyCompletedException thisShouldNotHappen) {
            log.error("SignatureProof could not be created : " + thisShouldNotHappen, thisShouldNotHappen);
        }

        if (certificateValid) {
        	return InvocationResponse.CONTINUE;
        } else {
        	return InvocationResponse.ABORT;
        }
    }

    @Override
    public void flowComplete(MessageContext arg0) {
        // Do nothing...
    }

    @Override
    public String getName() {
        return NAME;
    }

	@Override
    public Parameter getParameter(String name) {
        return this.handlerDesc.getParameter(name);
    }

    @Override
    public HandlerDescription getHandlerDesc() {
        return handlerDesc;
    }
}