/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and 
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis Søgaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */

package dk.gov.oiosi.common.cache;

import java.util.ArrayList;
import java.util.HashMap;

import dk.gov.oiosi.common.OutVariable;
import dk.gov.oiosi.exception.ArgumentNullException;

/**
 *  A cache implementation that stores a defined number of results in
 *  memory 
 */
public class QuantityCache<TKey, TValue> implements ICache<TKey, TValue> {
    private ArrayList<TKey> _indexSortedKeys;
    private HashMap<TKey, TValue> _keyedDictionary;
    private int _maxSize;
    private static Object _lockObject = new Object();

    /**
     * Constructor that takes the max size of the cache
     * @param maxSize
     */
    public QuantityCache(int maxSize) {
        if (maxSize < 0) throw new IllegalArgumentException("maxSize"); 
        _maxSize = maxSize;
        _indexSortedKeys = new ArrayList<TKey>(maxSize);
        _keyedDictionary = new HashMap<TKey, TValue>(maxSize);
    }

	@Override
	public void add(TKey key, TValue value) {
        if (key == null) throw new ArgumentNullException("key");
        if (_maxSize == 0) return;
        synchronized (_lockObject) {
            expire();
            _indexSortedKeys.add(key);
            _keyedDictionary.put(key, value);
        }
	}
	
	public void set(TKey key, TValue value) {
        add(key, value);
	}

	@Override
	public boolean containsKey(TKey key) {
        if (key == null) throw new ArgumentNullException("key");
        if (_maxSize == 0) return false;
        synchronized (_lockObject) {
            if (!_keyedDictionary.containsKey(key)) return false;
            updateIndex(key);
            return true;
        }
	}

	@Override
	public void remove(TKey key) {
        if (key == null) throw new ArgumentNullException("key");
        if (_maxSize == 0) return;
        synchronized (_lockObject) {
            if (!_keyedDictionary.containsKey(key)) return;
            _indexSortedKeys.remove(key);
            _keyedDictionary.remove(key);
        }
	}

	@Override
	public boolean tryGetValue(TKey key, OutVariable<TValue> value) {
        if (key == null) throw new ArgumentNullException("key");
        if (_maxSize == 0) return false;
        synchronized (_lockObject) {
        	TValue storedValue = _keyedDictionary.get(key); 
            if (storedValue == null) return false;
            updateIndex(key);
            value.setVariable(storedValue);
            return true;
        }	
    }
  
    private void updateIndex(TKey key) {
        _indexSortedKeys.remove(key);
        _indexSortedKeys.add(key);
    }

    private void expire() {
        if (_keyedDictionary.size() == 0) return;
        if (_keyedDictionary.size() < _maxSize) return;
        TKey key = _indexSortedKeys.get(0);
        _keyedDictionary.remove(key);
        _indexSortedKeys.remove(0);
    }
}
