/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis Søgaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *	 Christian Uldall Pedersen (christian.u.pedersen@accenture.com)
 */

package dk.gov.oiosi.common.cache;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;

import dk.gov.oiosi.common.OutVariable;
import dk.gov.oiosi.common.TimeSpan;

/**
 * Simple, generic timed cache type. The cache is traversed each time an element is inserted or extracted.
 * 
 */
public class TimedCache<TKey, TValue> implements ITimedCache<TKey, TValue> {

    /**
     * Lifetime of cache element.
     */
    private TimeSpan cacheTimeOut;

    /**
     * Internal map.
     */
    private Hashtable<TKey, RaspCacheValue<TValue>> cache;

    /**
     * TimedCache constructor.
     * @param t time span for cache
     */
    public TimedCache(final TimeSpan t) {
        cacheTimeOut = t;
        cache = new Hashtable<TKey, RaspCacheValue<TValue>>();
    }

    /**
     * TimedCache constructor.
     * @param t time span for cache
     */
    public TimedCache(final long t) {
    	this(new TimeSpan(t));
    }

    /**
     * Add to cache.
     * @param key search key
     * @param value object to cache
     */
    @Override
    public final void add(final TKey key, final TValue value) {
        long timeOut = System.currentTimeMillis() + cacheTimeOut.getSpanMillis();
        RaspCacheValue<TValue> cacheValue = new RaspCacheValue<TValue>(timeOut, value);
        synchronized (cache) {
            checkExpired();
            cache.put(key, cacheValue);
        }
    }
    
    /**
     * Sets the value of the specified key.
     */
    @Override
    public final void set(final TKey key, final TValue value) {
    	remove(key);
    	add(key, value);
    }

    /**
     * Get from cache. Return null if not in cache.
     * @param key search key
     * @param value the value found stored in an OutVariable
     * @return whether any results where found
     */
    @Override
    public final boolean tryGetValue(final TKey key, OutVariable<TValue> value) {
        synchronized (cache) {
            checkExpired();
            RaspCacheValue<TValue> cacheValue = cache.get(key);
            if (cacheValue == null) return false;
            value.setVariable(cacheValue.getValue());
            return true;
        }
    }

    /**
     * Checks whether the cache contains an element with the given key.
     * 
     * @param key The key to check
     * @return true if in cache
     */
    @Override
    public final boolean containsKey(final TKey key) {
        synchronized (cache) {
            checkExpired();
            return cache.containsKey(key);
        }
    }

    /**
     * Removes the given key from the cache.
     * 
     * @param key The key to remove
     */
    @Override
    public final void remove(final TKey key) {
        synchronized (cache) {
            checkExpired();
            cache.remove(key);
        }
    }

    /**
     * Check and remove expired entries.
     */
    private void checkExpired() {
        long now = System.currentTimeMillis();
        List<TKey> expiredList = new ArrayList<TKey>();
        Enumeration<TKey> enumeration = cache.keys();
        while (enumeration.hasMoreElements()) {
            TKey key = enumeration.nextElement();
            RaspCacheValue<TValue> pair = cache.get(key);
            if (pair.getTimeOut() < now) {
                expiredList.add(key);
            }
        }
        for (TKey key : expiredList) {
            cache.remove(key);
        }
    }

    /**
     * Internal cache entry class.
     */
    private class RaspCacheValue<T> {

        /**
         * timeout for entry.
         */
        private long timeOut;

        /**
         * cached element.
         */
        private T value;

        /**
         * Construct cache entry.
         * 
         * @param t
         *            timeout for entry
         * @param v
         *            value of entry
         */
        public RaspCacheValue(final long t, final T v) {
            timeOut = t;
            value = v;
        }

        /**
         * timeout for entry.
         * 
         * @return timeout
         */
        public final long getTimeOut() {
            return timeOut;
        }

        /**
         * Value.
         * 
         * @return value
         */
        public final T getValue() {
            return value;
        }
    }
}
