/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and 
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis Søgaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */
 
package dk.gov.oiosi.communication;

import org.apache.axiom.om.OMElement;
import org.apache.axis2.AxisFault;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.common.RaspLibraryException;
import dk.gov.oiosi.communication.client.IAxisDispatcher;
import dk.gov.oiosi.communication.client.ProxyGenerationException;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.security.SignatureValidationProof;
import dk.gov.oiosi.xml.XmlDocumentException;

/**
 * Represents a request to a RASP HTTP or email endpoint.
 */
public class Request implements IRequest {

    /**
     * Local logger.
     */
    private Log log = LogFactory.getLog(Request.class);

    /**
     * Credentials set by caller.
     */
    private Credentials credentials;

    /**
     * Request/Destination URI.
     */
    private Uri requestUri;

    /**
     * Policy.
     */
    private SendPolicy policy = null;

    /**
     * Default constructor.
     * 
     * @throws MainException
     */
    public Request() throws MainException {
    }

    /**
     * Constructor
     * 
     * @param endpointAddress
     *            endpoint URI
     */
    public Request(final Uri endpointAddress) throws MainException {
        // Test that a valid endpoint address was given
        testEndpointAddressCompability(endpointAddress.toString());
        setRequestUri(endpointAddress);
        setPolicy(ConfigurationHandler.getInstance().getSendPolicy());
    }

    /**
     * Constructor.
     * 
     * @param endpointAddress
     *            endpoint uri
     * @param credentials
     *            credentials
     * @throws MainException
     */
    public Request(final Uri endpointAddress, final Credentials credentials) throws MainException {
        this(endpointAddress);
        setCredentials(credentials);
        setPolicy(ConfigurationHandler.getInstance().getSendPolicy());
    }

    /**
     * Creates a new RaspRequest instance bound to the specific endpoint. During
     * construction, the type of endpoint is inferred from the Uri scheme. After
     * construction, additional properties may be set for the request, such as
     * credentials
     * 
     * @param credentials
     *            Overrides the credentials set in config
     * @param endpointAddress
     *            Endpoint uri
     * @param sendPolicy
     *            The send policy of the request
     * @throws MainException
     */
    public Request(final Uri endpointAddress, final Credentials credentials, final SendPolicy sendPolicy) throws MainException {
        this(endpointAddress, credentials);
        setPolicy(sendPolicy);
    }

    /**
     * modify RASP call not to use RM.
     */
    private boolean useRM = true;

    /**
     * Set whether the RASP call should use reliable messaging (RM).
     * 
     * @param useRM True, if the call should use RM
     */
    public void setUseRM(boolean useRM) {
        this.useRM = useRM;
    }

    /**
     * modify RASP call not to use WSS.
     */
    private boolean useWSS = true;

    /**
     * Set whether to use WS-Security in RASP call.
     * 
     * @param useWSS True, if the call should use WS-Security
     */
    public void setUseWSS(boolean useWSS) {
        this.useWSS = useWSS;
    }

    /**
     * use this proxyport in HTTP soap request.
     */
    private int proxyPort = -1;

    /**
     * Use this proxyport in HTTP soap request.
     * 
     * @param port The proxy port
     */
    public void setProxyPort(int port) {
        proxyPort = port;
    }

    /**
     * use this proxyhost in HTTP soap request.
     */
    private String proxyHost = null;

    /**
     * Use this proxyhost in HTTP soap request.
     * 
     * @param host The proxy host
     */
    public void setProxyHost(String host) {
        proxyHost = host;
    }

    /**
     * Fallback axis configuration file.
     */
    private String axisConfigurationFile = "./conf/client_axis2.xml";

    /**
     * Set the fallback axis configuration file.
     * 
     * @param value
     *            path to configuration file
     */
    public void setAxisConfigurationFile(String value) {
        axisConfigurationFile = value;
    }

    /**
     * Fallback axis repository folder.
     */
    private String axisRepositoryDir = "./test/conf/client";

    /**
     * Set the fallback axis repository folder.
     * 
     * @param value
     *            path to repository folder
     */
    public void setRepositoryDir(String value) {
        axisRepositoryDir = value;
    }

    /**
     * @deprecated The configuration directory was never used.
     * @param value
     *            configuration directory
     */
    public void setConfDir(String value) {}

    /**
     * Tests that the endpoint address is compatible with Rasp requests
     * 
     * @param endpointAddress
     *            endpoint URI
     * @throws RequestNoEndpointGivenException
     *             if input == null
     * @throws RequestNotSupportedSchemeException
     *             if unknown transport
     */
    protected void testEndpointAddressCompability(String endpointAddress) throws RequestNoEndpointGivenException,
            RequestNotSupportedSchemeException {

        if (endpointAddress == null)
            throw new RequestNoEndpointGivenException();

        // We only support 'http' and 'mailto'
        String scheme = endpointAddress.toLowerCase();
        if (!scheme.startsWith("mailto") && !scheme.startsWith("http"))
            throw new RequestNotSupportedSchemeException(scheme);
    }

    /**
     * The actual 'request'.
     */
    private IAxisDispatcher dispatcher = null;

    /**
     * Creates the proxy object. Note: This method is inherited from .Net...
     * 
     * @throws ProxyGenerationException
     */
    protected void createProxy() throws ProxyGenerationException {
        log.debug("Proxy being created");

        try {
            dispatcher = IAxisDispatcher.Factory.getDispatcher(axisConfigurationFile, axisRepositoryDir, useWSS, useRM, policy
                    .getWSSPolicyFileName(), policy.getHttpRMPolicy(), policy.getMailRMPolicy(), policy.getInboxMailConfiguration(), proxyPort,
                    proxyHost);
        } catch (Exception e) {
            log.error("Failed to crate Axis Dispatcher");
            throw new ProxyGenerationException(e);
        }
    }

    private void validateParameters(OiosiMessage message) throws MainException {
        // test endpoint address
        if(requestUri == null) {
            throw new MainException("Missing : endpoint address");
        }
        
        // test credentials
        if(credentials == null) {
            throw new MainException("Missing : credentials");
        }
        
        // test policy
        if(policy == null) {
            throw new MainException("Missing : policy");
        }

    }
    
    /**
     * Synchronously sends a request and gets a response
     * 
     * @param message The message to send
     * @return The response message
     * @throws MainException 
     */
    @Override
    public Response getResponse(OiosiMessage message) throws MainException {
    	log.debug("Send Message ");
    	Response response = null;
    	validateParameters(message);
    	createProxy();
    	response = sendMessage(message);
    	return response;
    }


    
    /**
     * Send message. Gets response. Handles exception.
     * 
     * @param message message
     * @return
     */
    private Response sendMessage(OiosiMessage message) throws MainException {
        Response response = null;
        OMElement soapReply;
    	SignatureValidationProof[] signatureProof = new SignatureValidationProof[1];
    	
        try {
            log.debug("Send request using Axis dispatcher");
            soapReply = dispatcher.sendRaspRequest(message.getMessageXml().getOMDocument(), getRequestUri().toString(), message
                    .getRequestAction(), credentials.getServerCertificate().getCertificate(), message.getMessageHeaders(), message.getProperties(), signatureProof);

            if (soapReply == null) {
                // this is OK, and means that the server replied with an empty SOAP message
                response = new Response(null, null);
            } else {
                response = new Response(soapReply, null);
            }
            
            if(signatureProof != null)
            	response.getResponseMessage().setSignatureProof(signatureProof[0]);

        } catch (XmlDocumentException e) {
            log.error("Failed to get document : " + e, e);
            throw e;
        } catch (AxisFault e) {
            log.error("Failed to get document : " + e, e);
            throw new OiosiCommunicationException(e.getMessage(), e);
        } catch (MainException e) {
            throw e;
        } catch (Exception e) {
            log.error("Failed to get document : " + e, e);
            throw new RaspLibraryException(e.getMessage(), e);
        }

        // 4. Return
        return response;
    }

    /**
     * Gets the credentials 
     * @return the credentials
     */
    @Override
    public Credentials getCredentials() {
        return credentials;
    }

    /**
     * Sets the credentials
     * @param value The new credentials for the request
     */
    @Override
    public void setCredentials(Credentials value) {
        credentials = value;
    }

    /**
     * Gets the request URI
     * @return the request URI
     */
    @Override
    public Uri getRequestUri() {
        return requestUri;
    }

    /**
     * Sets the request URI
     * @param u The request URI to use
     */
    public void setRequestUri(Uri u) {
        requestUri = u;
    }

    /**
     * Gets the send policy
     * @return the send policy
     */
    @Override
    public SendPolicy getPolicy() {
        return policy;
    }

    /**
     * Sets the send policy
     * @param value The send policy to use
     */
    @Override
    public void setPolicy(SendPolicy value) {
        policy = value;
        axisConfigurationFile = value.getAxisConfigurationFile();
        axisRepositoryDir = value.getAxisRepositoryDir();
    }
}
