/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and 
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis Søgaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */

package dk.gov.oiosi.communication.configuration;

import java.util.ArrayList;
import java.util.List;

import dk.gov.oiosi.exception.ArgumentNullException;

/**
 * This class represents a mapping between unique profile names and the corresponding tModel GUID.
 */
public class ProfileMappingCollectionConfig {

    private List<ProfileMapping> _profileMappings = new ArrayList<ProfileMapping>();

    /**
     * Returns a list of mappings between unique profile names and their 
     * corresponding tModel GUID
     * 
     * @return The mapping
     */
    public Iterable<ProfileMapping> getProfileMappingIterator() {	
    	return _profileMappings;
    }
    
    /**
     * Sets the collection of profile mappings
     * @param collection The collection to use in this configuration
     */
    public void setProfileMappings(ProfileMapping[] collection) {
    	_profileMappings.clear();
    	for (ProfileMapping mapping: collection) {
    		_profileMappings.add(mapping);
    	}
    }
    
    /**
     * Add the collection of profile mappings to the current list
     * @param collection The collection to add
     */
    public void addProfileMappings(ProfileMapping[] collection) {
    	for (ProfileMapping mapping: collection) {
    		_profileMappings.add(mapping);
    	}
    }

    /**
     * Adds a new profile mapping to the collection
     * @throws DocumentAllreadyAddedException 
     */
    public void AddProfileMapping(ProfileMapping profileMapping) throws DocumentAllreadyAddedException {
        if (profileMapping == null) throw new ArgumentNullException("profileMapping");
        if (containsProfileMappingByName(profileMapping.getName())) throw new DocumentAllreadyAddedException(profileMapping.getName());
        _profileMappings.add(profileMapping);
    }

    /**
     * Clears the mapping list
     */
    public void Clear() {
        if (_profileMappings != null) {
            _profileMappings.clear();
        }
    }

    /**
     * Determines whether the mapping collection contains a mapping with the name profileMappingName
     * 
     * @param profileMappingName The mapping name to check the existence of.
     * @return True, if profile mapping exists.
     */
    public boolean containsProfileMappingByName(String profileMappingName) {
    	for (ProfileMapping mapping: _profileMappings) {
    		String currentProfileMappingName = mapping.getName(); 
    		if (mapping.getName().equals(profileMappingName)) return true;
    	}
    	return false;
    }

    /**
     * Get the profile mapping corresponding to the given name
     * 
     * @param profileMappingName The profile mapping name
     * @return The corresponding profile mapping
     */
    public ProfileMapping getMapping(String profileMappingName)
    {
    	for (ProfileMapping mapping: _profileMappings) {
    		if (mapping.getName().equals(profileMappingName)) return mapping;
    	}
        return null;
    }
}