/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and 
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis Søgaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */

package dk.gov.oiosi.configuration;

import java.net.URISyntaxException;
import java.text.DateFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.configuration.SubnodeConfiguration;
import org.apache.commons.configuration.XMLConfiguration;
import org.apache.commons.configuration.reloading.FileChangedReloadingStrategy;
import org.apache.commons.configuration.tree.xpath.XPathExpressionEngine;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.addressing.EndpointAddress;
import dk.gov.oiosi.addressing.Identifier;
import dk.gov.oiosi.common.IdentifierUtility;
import dk.gov.oiosi.common.TimeSpan;
import dk.gov.oiosi.common.Version;
import dk.gov.oiosi.communication.MailAddress;
import dk.gov.oiosi.communication.RMPolicy;
import dk.gov.oiosi.communication.SendPolicy;
import dk.gov.oiosi.communication.Uri;
import dk.gov.oiosi.communication.configuration.CustomHeaderConfiguration;
import dk.gov.oiosi.communication.configuration.CustomHeaderXPathConfiguration;
import dk.gov.oiosi.communication.configuration.KeyTypeMapping;
import dk.gov.oiosi.communication.configuration.KeyTypeMappingExpression;
import dk.gov.oiosi.communication.configuration.DocumentEndpointInformation;
import dk.gov.oiosi.communication.configuration.DocumentTypeCollectionConfig;
import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.communication.configuration.ProfileMapping;
import dk.gov.oiosi.communication.configuration.ProfileMappingCollectionConfig;
import dk.gov.oiosi.communication.configuration.ServiceEndpointFriendlyName;
import dk.gov.oiosi.communication.configuration.ServiceEndpointKey;
import dk.gov.oiosi.communication.configuration.mail.IMailServerConfiguration;
import dk.gov.oiosi.communication.configuration.mail.MailAuthenticationMode;
import dk.gov.oiosi.communication.configuration.mail.MailServerConfiguration;
import dk.gov.oiosi.communication.configuration.mail.MailServerConnectionPolicy;
import dk.gov.oiosi.security.CertificateSubject;
import dk.gov.oiosi.security.RootCertificateConfig;
import dk.gov.oiosi.security.ldap.LdapLookupFactoryConfig;
import dk.gov.oiosi.security.ldap.LdapSettings;
import dk.gov.oiosi.security.oces.InvalidOcesCertificateSubjectKeyException;
import dk.gov.oiosi.security.oces.OcesCertificateSubjectKey;
import dk.gov.oiosi.security.oces.OcesX509CertificateConfig;
import dk.gov.oiosi.security.revocation.RevocationLookupFactoryConfig;
import dk.gov.oiosi.security.revocation.ocsp.OcspConfig;
import dk.gov.oiosi.uddi.LookupRegistryFallbackConfig;
import dk.gov.oiosi.uddi.Registry;
import dk.gov.oiosi.uddi.RegistryLookupClientFactoryConfig;
import dk.gov.oiosi.uddi.UddiConfig;
import dk.gov.oiosi.uddi.UddiId;
import dk.gov.oiosi.uddi.UddiLookupClientFactoryConfig;
import dk.gov.oiosi.uddi.UddiLookupResponse;
import dk.gov.oiosi.uddi.ProcessRoleDefinition;
import dk.gov.oiosi.xml.schematron.SchematronStoreConfig;
import dk.gov.oiosi.xml.schematron.SchematronValidationConfig;
import dk.gov.oiosi.xml.xpath.PrefixedNamespace;
import dk.gov.oiosi.xml.xpath.discriminator.XPathDiscriminatorConfig;
import dk.gov.oiosi.xml.xpath.discriminator.XPathDiscriminatorConfigCollection;


/**
 * ReadOnly version of RaspConfigurationHandler.
 * 
 * Reads RaspConfiguration.xml - and initializes configuration classes.
 */
public class ConfigurationHandler {
    
	/**
     * The default file name of the configuration file.
     */
    public static final String RASPCONFIGURATION_XML_DEFAULT = "RaspConfiguration.xml";
    
    private static String raspConfigurationFileName = RASPCONFIGURATION_XML_DEFAULT;
    private static ConfigurationHandler rcf = null;
    private Log log = LogFactory.getLog(ConfigurationHandler.class);
    private XMLConfiguration raspConfigurationFile = null;
    private LdapSettings ldapSettings = null;
    private OcspConfig ocspConfig = null;
    private UddiConfig uddiConfig = null;
    private LdapLookupFactoryConfig ldapFactory = null;
    private RevocationLookupFactoryConfig revocationFactory = null;
    private UddiLookupClientFactoryConfig uddiClientFactory = null;
    private RegistryLookupClientFactoryConfig registryClientFactory = null;
    private ProfileMappingCollectionConfig profileMappingCollectionConfig;
    private DocumentTypeCollectionConfig raspDocumentTypeCollectionConfig;
    private SendPolicy sendPolicy = null;
    private SchematronStoreConfig schematronStoreConfig = null;

    /**
     * Enforces the singleton pattern
     */
    private ConfigurationHandler() {}
    
    /**
     * Returns the static instance of the configuration handler.
     * 
     * @return ConfigurationHandler The static instance of the configuration handler
     */
    public synchronized static ConfigurationHandler getInstance() {
        if (rcf == null) {
            rcf = new ConfigurationHandler();
            rcf.loadXMLConfiguration();
        }
        return rcf;
    }

    /**
     * Sets the file name of the configuration file.
     * 
     * @param file The file name
     */
    public static void setConfigurationFile(String file) {
        raspConfigurationFileName = file;
    }

    /**
     * Loads the configuration from the file specified with setConfigurationFile(String file).
     */
    public synchronized void loadXMLConfiguration() {
        try {
            log.debug("Looking for configurationfile " + raspConfigurationFileName);
            try {
                raspConfigurationFile = new XMLConfiguration(raspConfigurationFileName);
            } catch (Exception e) {
                raspConfigurationFile = new XMLConfiguration("conf/" + raspConfigurationFileName);
            }
            raspConfigurationFile.setReloadingStrategy(new FileChangedReloadingStrategy());
        	raspConfigurationFile.setExpressionEngine(new XPathExpressionEngine());
        } catch (Exception e) {
            log.fatal(raspConfigurationFileName + " can not be used due to: " + e.getMessage());
            e.printStackTrace();
        }
    }

    /**
     * @deprecated
     * LDAP factory configuration
     * 
     * @return LDAP factory configuration
     */
    public LdapSettings getLDAPSettings() {
        return getLdapSettings();
    }

    /**
     * Get the LDAP Settings.
     * 
     * @return LDAP settings
     */
    public LdapSettings getLdapSettings() {
    	// The XPath for our configuration section
    	String xpathBase = "//ConfigurationSection[@*=\"LdapSettings\"]/";
    	
        log.debug("RaspConfigurationHandler . getLDAPSettings " + raspConfigurationFile);
        synchronized (raspConfigurationFile) {
            if (ldapSettings == null) {

                ldapSettings = new LdapSettings();

                ldapSettings.setHost(raspConfigurationFile.getString(xpathBase + "Host"));
                ldapSettings.setMaxResults(raspConfigurationFile.getInt(xpathBase + "MaxResults"));
                ldapSettings.setPort(raspConfigurationFile.getInt(xpathBase + "Port"));
                ldapSettings.setSearchServerTimeoutMsec(raspConfigurationFile
                        .getInt(xpathBase + "SearchServerTimeoutMsec"));
                ldapSettings.setSearchClientTimeoutMsec(raspConfigurationFile.getInt(xpathBase + "ConnectionTimeoutMsec"));
            }
        }
        return ldapSettings;
    }

    /**
     * Get the OCSP settings.
     * 
     * @return OCSP settings
     * @throws URISyntaxException 
     */
    public OcspConfig getOcspConfig() throws URISyntaxException{
    	// The XPath for our configuration section
    	String xpathBase = "//ConfigurationSection[@*=\"OcspConfig\"]/";
    	
        log.debug("RaspConfigurationHandler . getOcspConfig ");
        synchronized (raspConfigurationFile) {
            if (ocspConfig == null) {
                ocspConfig = new OcspConfig();
                ocspConfig.setDefaultTimeoutMsec(raspConfigurationFile.getInt(xpathBase + "DefaultTimeoutMsec"));
                
               	String serverUrl = raspConfigurationFile.getString(xpathBase + "ServerUrl");
               	if(serverUrl!=null)
               		ocspConfig.setServerUrl(serverUrl);
            }
        }
        return ocspConfig;
    }
    
    /**
     * Get the Root certificate settings
     * 
     * @return Root certificate settings
     */
    public RootCertificateConfig getRootCertificateConfig() {
    	// The XPath for our configuration section
    	String xpathBase = "//ConfigurationSection[@*=\"RootCertificateConfig\"]/";
    	RootCertificateConfig rootCertConfig = new RootCertificateConfig();
    	
        log.debug("RaspConfigurationHandler . getRootCertificateConfig ");
        synchronized (raspConfigurationFile) {        	
        	rootCertConfig.setKeyStoreLocation(raspConfigurationFile.getString(xpathBase + "KeyStoreLocation"));
        	rootCertConfig.setKeyStorePassword(raspConfigurationFile.getString(xpathBase + "KeyStorePassword"));
        	rootCertConfig.setKeyLabel(raspConfigurationFile.getString(xpathBase + "KeyLabel"));        	
        }
        return rootCertConfig;
    }
    
    /**
     * Get the OcesX509 certificate settings
     * 
     * @return OcesX509 certificate settings
     * @throws InvalidOcesCertificateSubjectKeyException
     */
    public OcesX509CertificateConfig getOcesX509CertificateConfig() throws InvalidOcesCertificateSubjectKeyException {
    	// The XPath for our configuration section
    	String xpathBase = "//ConfigurationSection[@*=\"OcesX509CertificateConfig\"]/";
    	OcesX509CertificateConfig ocesX509CertConfig = new OcesX509CertificateConfig();
    	
        log.debug("RaspConfigurationHandler . getOcesX509CertificateConfig ");
        synchronized (raspConfigurationFile) {        	
        	ocesX509CertConfig.set_personalCertificateSubjectKey(new OcesCertificateSubjectKey(raspConfigurationFile.getString(xpathBase + "/PersonalCertificateSubjectKey/SubjectKeyString")));
        	ocesX509CertConfig.set_organizationCertificateSubjectKey(new OcesCertificateSubjectKey(raspConfigurationFile.getString(xpathBase + "/OrganizationCertificateSubjectKey/SubjectKeyString")));
        	ocesX509CertConfig.set_employeeCertificateSubjectKey(new OcesCertificateSubjectKey(raspConfigurationFile.getString(xpathBase + "/EmployeeCertificateSubjectKey/SubjectKeyString")));
        	ocesX509CertConfig.set_functionCertificateSubjectKey(new OcesCertificateSubjectKey(raspConfigurationFile.getString(xpathBase + "/FunctionCertificateSubjectKey/SubjectKeyString")));
        }
        return ocesX509CertConfig;
    }
    
    /**
     * Gets the LDAP test settings.
     * 
     * @return LDAP test settings
     */
    public String getLdapTestConfig() {
    	// The XPath for our configuration section
    	String xpathBase = "//ConfigurationSection[@*=\"LdapCertificateLookupTestConfig\"]/";
    	return raspConfigurationFile.getString(xpathBase + "FileName");
    }
    
    /**
     * Gets the revocation test settings.
     * 
     * @return Revocation test settings
     */
    public boolean getRevocationTestConfig() {
    	// The XPath for our configuration section
    	String xpathBase = "//ConfigurationSection[@*=\"RevocationLookupTestConfig\"]/";

    	boolean response = false;
        synchronized (raspConfigurationFile) {
        	response = raspConfigurationFile.getBoolean(xpathBase + "ReturnPositiveResponse");
        }
        return response;
    }
    
    /**
     * Get the UDDI settings.
     * 
     * @return UDDI settings
     * @throws URISyntaxException 
     */
    public UddiConfig getUddiConfig() throws URISyntaxException {
    	// The XPath for our configuration section
    	String xpathBase = "//ConfigurationSection[@*=\"UddiConfig\"]/";
    	
    	log.debug("RaspConfigurationHandler . getUddiConfig ");
        synchronized (raspConfigurationFile) {
        	if (uddiConfig != null) return uddiConfig;
            uddiConfig = new UddiConfig();

            uddiConfig.setProfileConformanceClaim(raspConfigurationFile.getString(xpathBase + "ProfileConformanceClaim"));
            uddiConfig.setRegistrationConformanceClaim(raspConfigurationFile.getString(xpathBase + "RegistrationConformanceClaim"));
            uddiConfig.setPublishEndpoint(raspConfigurationFile.getString(xpathBase + "PublishEndpoint"));
            uddiConfig.setSecurityEndpoint(raspConfigurationFile.getString(xpathBase + "SecurityEndpoint"));
            uddiConfig.setFallbackTimeoutMinutes(raspConfigurationFile.getInt(xpathBase + "FallbackTimeoutMinutes"));
            SubnodeConfiguration lookupRegistryFallbackSubnodes = raspConfigurationFile.configurationAt(xpathBase + "LookupRegistryFallbackConfig");
            LookupRegistryFallbackConfig lookupRegistryFalbackConfig = getLookupRegistryFallbackConfig(lookupRegistryFallbackSubnodes);
            uddiConfig.setLookupRegistryFallbackConfig(lookupRegistryFalbackConfig);
        }
        return uddiConfig;
    }
    
    private LookupRegistryFallbackConfig getLookupRegistryFallbackConfig(SubnodeConfiguration lookupRegistryFallbackSubnodes) throws URISyntaxException {
    	LookupRegistryFallbackConfig config = new LookupRegistryFallbackConfig();
    	ArrayList<Registry> registries = new ArrayList<Registry>();
    	List<?> registrySubnodes = lookupRegistryFallbackSubnodes.configurationsAt("PrioritizedRegistryList/Registry");
    	for (Object registrySubnodeObject : registrySubnodes) {
			SubnodeConfiguration registrySubnode = (SubnodeConfiguration)registrySubnodeObject;
			Registry registry = getRegistry(registrySubnode);
			registries.add(registry);
		}
     	config.setRegistries(registries);
     	return config;
    }
    
    
    private Registry getRegistry(SubnodeConfiguration registrySubnode) throws URISyntaxException {
    	String xpath = "/EndpointCollection/Endpoint"; 
    	String[] endpoints = registrySubnode.getStringArray(xpath);
    	Registry registry = new Registry(endpoints);
    	return registry;
    }
    
    /**
     * Gets the document type configurations.
     * 
     * @return Document type configurations
     * @throws ConfigurationException
     */
    public DocumentTypeCollectionConfig getRaspDocumentTypeCollectionConfig() throws ConfigurationException {
    	log.debug("RaspConfigurationHandler . getRaspDocumentTypeCollectionConfig ");
        synchronized (raspConfigurationFile) {
            if (raspDocumentTypeCollectionConfig == null) {
                try {
                    raspDocumentTypeCollectionConfig = new DocumentTypeCollectionConfig();

                    String[] docs = raspConfigurationFile
                            .getStringArray("//DocumentTypes/DocumentTypeConfig/RootName");
                    for (int i = 0; i < docs.length; i++) {
                        DocumentTypeConfig typeConfig = initDocumentTypeConfig(i);
                        raspDocumentTypeCollectionConfig.addDocumentType(typeConfig);
                    }
                } catch (Exception e) {
                    ConfigurationException ex = new ConfigurationException("Could not init DocumentTypeConfig " + e);
                    ex.setStackTrace(e.getStackTrace());
                    throw ex;
                }
            }
        }
        return raspDocumentTypeCollectionConfig;
    }

    /**
     * Gets the send policy.
     * 
     * @return The send policy
     * @throws ConfigurationException
     */
    public SendPolicy getSendPolicy() throws ConfigurationException {
    	// The XPath for our configuration section
    	String xpathBase = "//ConfigurationSection[@*=\"Axis2JavaRasp\"]/";
    	
    	String xpathBaseMail = "//ConfigurationSection[@*=\"EmailTransportUserConfig\"]/";
    	
    	log.debug("RaspConfigurationHandler . getSendPolicy");
        synchronized (raspConfigurationFile) {
            if (sendPolicy == null) {
                sendPolicy = new SendPolicy();

                String wssPolicy = raspConfigurationFile.getString(xpathBase + "WSSPolicyFile");
                log.debug("- wss policy file : " + wssPolicy);
                sendPolicy.setWSSPolicyFileName(wssPolicy);

                String axisConf = raspConfigurationFile.getString(xpathBase + "AxisConfigurationFile");
                log.debug("- axis configuration : " + axisConf);
                sendPolicy.setAxisConfigurationFile(axisConf);

                String repoDir = raspConfigurationFile.getString(xpathBase + "AxisRepositoryDir");
                log.debug("- axis repositorty dir : " + repoDir);
                sendPolicy.setAxisRepositoryDir(repoDir);
                
                String mailPrefix = xpathBase + "RMPolicy/MailTransport/RMAssertion/Policy/";
                RMPolicy mailPolicy = initRMPolicy(mailPrefix);
                sendPolicy.setMailRMPolicy(mailPolicy);
                
                String httpPrefix = xpathBase + "RMPolicy/HTTPTransport/RMAssertion/Policy/";
                RMPolicy httpPolicy = initRMPolicy(httpPrefix);
                sendPolicy.setHttpRMPolicy(httpPolicy);
                
                String sendInbox = xpathBaseMail + "SendInbox/";
                IMailServerConfiguration mailConfig = initMailConfiguration(sendInbox);
                sendPolicy.setInboxMailConfiguration(mailConfig);
            }
            return sendPolicy;
        }
    }

	/**
     * Gets the UDDI factory configuration.
     * 
     * @return UDDI factory configuration
     */
    public UddiLookupClientFactoryConfig getUddiLookupClientFactoryConfig() {
        log.debug("RaspConfigurationHandler . getUddiLookupClientFactoryConfig ");
        synchronized (raspConfigurationFile) {
            if(uddiClientFactory==null) {
                initFactories();
            }
        }
        return uddiClientFactory;
    }

    /**
     * Gets the Registry lookup factory configuration.
     * 
     * @return Registry factory configuration
     */
    public RegistryLookupClientFactoryConfig getRegistryLookupClientFactoryConfig() {
        log.debug("RaspConfigurationHandler . getUddiLookupClientFactoryConfig ");
        synchronized (raspConfigurationFile) {
            if(registryClientFactory==null) {
                initFactories();
            }
        }
        return registryClientFactory;
    }

    
    /**
     * Gets the UDDI client test configuration
     * @return UDDI client test configuration
     * @throws ParseException
     * @throws URISyntaxException 
     */
    public UddiLookupResponse getUddiClientTestConfig() throws ParseException, URISyntaxException {
    	// The XPath for our configuration section
    	String xpathBase = "//ConfigurationSection[@*=\"UddiLookupClientTestConfig\"]/";
    	
    	synchronized(raspConfigurationFile){
	    	 	    	
	    	// Reads the configuration elements from file
	    	Identifier endpointKey= null;
	    	EndpointAddress endpointAddress= IdentifierUtility.getEndpointAddressFromString(raspConfigurationFile.getString(xpathBase + "EndpointAddress"));
	    	Date activationUTC= DateFormat.getDateInstance().parse(raspConfigurationFile.getString(xpathBase + "ActivationDate"));
	    	Date expirationUTC= DateFormat.getDateInstance().parse(raspConfigurationFile.getString(xpathBase + "ExpirationDate"));
	    	String cert= raspConfigurationFile.getString(xpathBase + "CertificateSubjectSerialNumber");
	    	Uri termsOfUse= new Uri(raspConfigurationFile.getString(xpathBase + "TermsOfUseUrl"));
	    	MailAddress contactMail= new MailAddress(raspConfigurationFile.getString(xpathBase + "ServiceContactEmail"));
	    	Version version= new Version(raspConfigurationFile.getString(xpathBase + "Version"));
	    	UddiId newerVersion= null;
	    	if(raspConfigurationFile.getBoolean(xpathBase + "HasNewerVersion")){
	    		newerVersion = IdentifierUtility.getUddiIDFromString(raspConfigurationFile.getString(xpathBase + "NewerVersionReference"));
	    	}
	    	
	    	// Returns the UddiLookupResponse
			return new UddiLookupResponse(endpointKey, endpointAddress, activationUTC, expirationUTC, new CertificateSubject(cert), termsOfUse, contactMail, version, newerVersion, new ArrayList<ProcessRoleDefinition>());
    	}
    }

    /**
     * Gets the LDAP factory configuration
     * 
     * @return LDAP factory configuration
     */
    public LdapLookupFactoryConfig getLdapLookupFactoryConfig() {
        log.debug("RaspConfigurationHandler . getLdapLookupFactoryConfig ");
        synchronized (raspConfigurationFile) {
            if(ldapFactory==null) {
                initFactories();
            }
        }
        return ldapFactory;
    }

    /**
     * Gets the revocation lookup client factory
     * 
     * @return Revocation lookup client factory
     */
    public RevocationLookupFactoryConfig getRevocationLookupFactoryConfig() {
        log.debug("RaspConfigurationHandler . getRevocationLookupFactoryConfig ");
        synchronized (raspConfigurationFile) {
            if(revocationFactory==null) {
                initFactories();
            }
        }
        return revocationFactory;
    }
    
    
    /**
     * Gets the ProfileMapping configuration.
     */
    public ProfileMappingCollectionConfig getProfileMappingCollectionConfig() throws ConfigurationException {
        synchronized (raspConfigurationFile) {
            if (profileMappingCollectionConfig == null) {
                try {
                	profileMappingCollectionConfig = new ProfileMappingCollectionConfig();

                    String[] names = raspConfigurationFile.getStringArray("//ProfileMappingCollection/ProfileMapping/Name");
                    String[] tModelIds = raspConfigurationFile.getStringArray("//ProfileMappingCollection/ProfileMapping/TModelGuid");
                    if (names != null && tModelIds != null && names.length != tModelIds.length ) {
                    	throw new ConfigurationException("ProfileMappingCollectionConfig: Different number of names and tModelGuids");
                    }
                    	
                    int index = 0;
                    for (String name: names) {
                        ProfileMapping profileMapping = new ProfileMapping();
                        profileMapping.setName(name);
                        profileMapping.setTModelGuid(tModelIds[index]);
                        profileMappingCollectionConfig.AddProfileMapping(profileMapping);
                        index ++;
                    }
                }
                catch (Exception e) {
                    ConfigurationException ex = new ConfigurationException("Could not init ProfileMappingCollectionConfig " + e);
                    ex.setStackTrace(e.getStackTrace());
                    throw ex;
                }
            }
        }
        return profileMappingCollectionConfig;
    }
    
    /**
     * Gets the schematron store configuration from configuration file.
     * @return The schematron store configuration
     */
    public SchematronStoreConfig getSchematronStoreConfig() {
    	synchronized (raspConfigurationFile) {
    		if (schematronStoreConfig != null) return schematronStoreConfig;
    		schematronStoreConfig = new SchematronStoreConfig();
    		
    		String xpathBase = "//ConfigurationSection[@*=\"SchematronStoreConfig\"]/";
    		int maxCompiledStylesheetsInMemory = raspConfigurationFile.getInt(xpathBase + "MaxCompiledStylesheetsInMemory");
    		schematronStoreConfig.setMaxCompiledStylesheetsInMemory(maxCompiledStylesheetsInMemory);
    		return schematronStoreConfig;
		}
    }
    
    /**
     * Initiate the reliable messaging configuration.
     * 
     * @param xpath
     *            where to find the configuration in the XML-document
     * @throws ConfigurationException
     */
    private RMPolicy initRMPolicy(String xpath) throws ConfigurationException {
        try {
            boolean eb = raspConfigurationFile.getBoolean(xpath + "ExponentialBackoff");
            long retr = raspConfigurationFile.getLong(xpath + "RetransmissionInterval");
            int max = raspConfigurationFile.getInt(xpath + "MaximumRetransmissionCount");

            RMPolicy policy = new RMPolicy(max, retr, eb);
            return policy;
        } catch (Exception e) {
            ConfigurationException ex = new ConfigurationException("Could not init SendPolicy " + e);
            ex.setStackTrace(e.getStackTrace());
            throw ex;
        }
    }
    
    /**
     * Initiate the a mail configuration.
     * 
     * @param xpath Where to find the configuration in the XML-document
     * @return The mail configuration given by the xpath.
     */
    private IMailServerConfiguration initMailConfiguration(String xpath) throws ConfigurationException {
    	try {
    		String serverAddress = raspConfigurationFile.getString(xpath + "ServerAddress");
			String password = raspConfigurationFile.getString(xpath + "Password");
			String userName = raspConfigurationFile.getString(xpath + "UserName");
			String replyAddress = raspConfigurationFile.getString(xpath + "ReplyAddress");
			TimeSpan pollingInterval = new TimeSpan(1000 * raspConfigurationFile.getInt(xpath + "ConnectionPolicy/PollingInterval"));
			MailAuthenticationMode authenticationMode = MailAuthenticationMode.valueOf(raspConfigurationFile.getString(xpath + "ConnectionPolicy/AuthenticationMode"));
			int port = raspConfigurationFile.getInt(xpath + "ConnectionPolicy/Port/Number");
			
			MailServerConnectionPolicy connectionPolicy = new MailServerConnectionPolicy(pollingInterval, authenticationMode, port);
			IMailServerConfiguration configuration = new MailServerConfiguration(serverAddress, userName, password, replyAddress, connectionPolicy );
		
			return configuration;
    	} catch (Exception e) {
            ConfigurationException ex = new ConfigurationException("Could not init MailConfiguration " + e);
            ex.setStackTrace(e.getStackTrace());
            throw ex;
        }
    }

    /**
     * Read document.
     * 
     * @param docIndex
     *            ix in configuration file
     * @return configuration for document
     * @throws Exception
     */
    private DocumentTypeConfig initDocumentTypeConfig(int docIndex) throws Exception {
    	// The XPath for our configuration section
    	String xpathBase = "//ConfigurationSection[@*=\"DocumentTypeCollectionConfig\"]/";
    	
        DocumentTypeConfig rd = new DocumentTypeConfig();

        log.debug("Retrieving basic data...");

        String documentSelect = xpathBase + "DocumentTypes/DocumentTypeConfig[" + (docIndex + 1) + "]";

        rd.setRootName(raspConfigurationFile.getString(documentSelect + "/RootName"));
        rd.setFriendlyName(raspConfigurationFile.getString(documentSelect + "/FriendlyName"));
        rd.setRootNamespace(raspConfigurationFile.getString(documentSelect + "/RootNamespace"));
        rd.setSchemaPath(raspConfigurationFile.getString(documentSelect + "/SchemaPath"));

        rd.setServiceContractTModel(raspConfigurationFile.getString(documentSelect + "/ServiceContractTModel"));
        rd.setStylesheetPath(raspConfigurationFile.getString(documentSelect + "/StylesheetPath"));

        String tmp = raspConfigurationFile.getString(documentSelect + "/XsltTransformStylesheetPath");
        if (tmp != null && tmp.length() > 0) {
            rd.setXsltTransformStylesheetPath(tmp);
        }

        //
        SchematronValidationConfig svc = new SchematronValidationConfig();
        svc.setSchematronDocumentPath(raspConfigurationFile.getString(documentSelect
                + "/SchematronValidationConfig/SchematronDocumentPath"));
        svc.setErrorXPath(raspConfigurationFile.getString(documentSelect + "/SchematronValidationConfig/ErrorXPath"));
        svc.setErrorMessageXPath(raspConfigurationFile.getString(documentSelect + "/SchematronValidationConfig/ErrorMessageXPath"));
        rd.setSchematronValidationConfig(svc);

        //
        DocumentEndpointInformation rdept = new DocumentEndpointInformation();
        rdept.setEndpointFriendlyName(new ServiceEndpointFriendlyName(raspConfigurationFile.getString(documentSelect
                + "/EndpointType/EndpointFriendlyName/XPath")));
        rdept.setSenderFriendlyName(new ServiceEndpointFriendlyName(raspConfigurationFile.getString(documentSelect
                + "/EndpointType/SenderFriendlyName/XPath")));

        rdept.setReplyAction(raspConfigurationFile.getString(documentSelect + "/EndpointType/ReplyAction"));
        rdept.setRequestAction(raspConfigurationFile.getString(documentSelect + "/EndpointType/RequestAction"));

        // Receiver Key
        ServiceEndpointKey receipientRdk = new ServiceEndpointKey();
        receipientRdk.setXPath(raspConfigurationFile.getString(documentSelect + "/EndpointType/Key/Xpath"));

        KeyTypeMappingExpression recipientKeyTypeMapping = getKeyTypeMappingExpression(documentSelect + "/EndpointType/Key");
        receipientRdk.addMappingExpression(recipientKeyTypeMapping);

        // Sender Key
        ServiceEndpointKey senderRdk = new ServiceEndpointKey();
        senderRdk.setXPath(raspConfigurationFile.getString(documentSelect + "/EndpointType/SenderKey/Xpath"));

        KeyTypeMappingExpression senderKeyTypeMapping = getKeyTypeMappingExpression(documentSelect + "/EndpointType/SenderKey");
        senderRdk.addMappingExpression(senderKeyTypeMapping);

        rdept.setKey(receipientRdk);
        rdept.setSenderKey(senderRdk);

        rd.setEndpointType(rdept);
        
        // Set ProfileId XPath
        String profileIdXPath = raspConfigurationFile.getString(documentSelect + "/EndpointType/ProfileIdXPath/XPath");
        rd.setProfileIdXPath(profileIdXPath);
        // End Set ProfileId XPath
        
        //
        int noOfEndNameSpaces = raspConfigurationFile.getList(documentSelect + "/Namespaces/PrefixedNamespace/Namespace").size();
        List<PrefixedNamespace> rdns = new ArrayList<PrefixedNamespace>();
        for (int x = 0; x < noOfEndNameSpaces; x++) {
            PrefixedNamespace rdn = new PrefixedNamespace();
            rdn.setPrefix(raspConfigurationFile.getString(documentSelect + "/Namespaces/PrefixedNamespace[" + (x+1) + "]/Prefix"));
            rdn.setNamespace(raspConfigurationFile.getString(documentSelect + "/Namespaces/PrefixedNamespace[" + (x+1) + "]/Namespace"));
            rdns.add(rdn);
        }
        rd.setNamespaces(rdns.iterator());

        String csXpath = documentSelect + "/CustomHeaderConfiguration/XPaths";

        CustomHeaderConfiguration chc = new CustomHeaderConfiguration();
        List<CustomHeaderXPathConfiguration> headerList = new ArrayList<CustomHeaderXPathConfiguration>();
        
        for(Iterator<?> it = raspConfigurationFile.getKeys(csXpath); it.hasNext(); )
        {
        	String key = it.next().toString();
            String xpath = raspConfigurationFile.getString(key);
            if(xpath.length() > 0)
            {
            	String elementName = key.substring(csXpath.length() + 1);
            	
            	CustomHeaderXPathConfiguration chxc = new CustomHeaderXPathConfiguration();
            	chxc.setName(elementName);
                chxc.setXPath(xpath);

                headerList.add(chxc);
            }
        }
        
        if (headerList.size() > 0)
        {
	        chc.setXPaths((CustomHeaderXPathConfiguration[]) headerList.toArray(new CustomHeaderXPathConfiguration[headerList.size()]));
	        rd.setCustomHeaderConfiguration(chc);
        }

        // xpath discriminator collection
        String[] identifierList = raspConfigurationFile.getStringArray(documentSelect
                + "/IdentifierDiscriminators/XPathDiscriminatorConfig/XPathExpression");
        if (identifierList.length > 0) {
            XPathDiscriminatorConfigCollection xpathIdentifiers = new XPathDiscriminatorConfigCollection();
            for (int i = 0; i < identifierList.length; i++) {
                XPathDiscriminatorConfig identifier = new XPathDiscriminatorConfig();

                String expression = raspConfigurationFile.getString(documentSelect
                        + "/IdentifierDiscriminators/XPathDiscriminatorConfig[" + (i+1) + "]/XPathExpression");
                String expectedResult = raspConfigurationFile.getString(documentSelect
                        + "/IdentifierDiscriminators/XPathDiscriminatorConfig[" + (i+1) + "]/XPathExpectedResult");
                identifier.setXPathExpression(expression);
                identifier.setXPathExpectedResult(expectedResult);

                xpathIdentifiers.add(identifier);
            }
            rd.setIdentifierDiscriminators(xpathIdentifiers);
        }

        log.debug("Finished retrieving document");

        return rd;
    }

    /**
     * Utility to initiate key type for sender/receiver key (CVR, EAN...).
     * 
     * @param path
     *            path in document
     * @return mapping
     */
    private KeyTypeMappingExpression getKeyTypeMappingExpression(String path) {
        KeyTypeMappingExpression keyTypeMapping = new KeyTypeMappingExpression();
        String keyTypeMappingXPath = path + "/MappingExpressions/KeyTypeMappingExpressions/Name";

        String[] mappings = raspConfigurationFile.getStringArray(keyTypeMappingXPath);
        for (int i = 0; i < mappings.length; i++) {
            String mapExprKey = path + "/MappingExpressions/KeyTypeMappingExpressions[" + (i+1) + "]";
            String mapName = raspConfigurationFile.getString(mapExprKey + "/Name");
            String mapXPath = raspConfigurationFile.getString(mapExprKey + "/XPathExpression");
            keyTypeMapping.setName(mapName);
            keyTypeMapping.setXPathExpression(mapXPath);
            String[] mapList = raspConfigurationFile.getStringArray(mapExprKey + "/Mappings/Mapping/Value");
            for (int j = 0; j < mapList.length; j++) {
                String mapListKey = mapExprKey + "/Mappings/Mapping[" + (j+1) + "]";
                String value = raspConfigurationFile.getString(mapListKey + "/Value");
                String mapsTo = raspConfigurationFile.getString(mapListKey + "/MapsTo");

                KeyTypeMapping map = new KeyTypeMapping();
                map.setValue(value);
                map.setMapsTo(mapsTo);

                keyTypeMapping.addMapping(map);
            }
        }
        return keyTypeMapping;
    }

    private void initFactories() {
    	registryClientFactory = new RegistryLookupClientFactoryConfig();
    	registryClientFactory.setImplementationNamespaceClass(raspConfigurationFile.getString("//ConfigurationSection[@*=\"RegistryLookupClientFactoryConfig\"]/ImplementationNamespaceClass"));
    	
    	uddiClientFactory = new UddiLookupClientFactoryConfig();
        uddiClientFactory.setImplementationNamespaceClass(raspConfigurationFile.getString("//ConfigurationSection[@*=\"UddiLookupClientFactoryConfig\"]/ImplementationNamespaceClass"));
        
        ldapFactory = new LdapLookupFactoryConfig();
        ldapFactory.setImplementationNamespaceClass(raspConfigurationFile.getString("//ConfigurationSection[@*=\"LdapLookupFactoryConfig\"]/ImplementationNamespaceClass"));
        
        revocationFactory = new RevocationLookupFactoryConfig();
        revocationFactory.setImplementationNamespaceClass(raspConfigurationFile.getString("//ConfigurationSection[@*=\"RevocationLookupFactoryConfig\"]/ImplementationNamespaceClass"));
        
    }
}
