/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis Søgaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */

package dk.gov.oiosi.security.ldap;

import java.io.ByteArrayInputStream;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.Collection;

import javax.naming.CommunicationException;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.SearchResult;
import javax.naming.ldap.InitialLdapContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.common.OutVariable;
import dk.gov.oiosi.common.cache.ITimedCache;
import dk.gov.oiosi.common.cache.TimedCache;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.exception.ArgumentNullException;
import dk.gov.oiosi.security.CertificateSubject;
import dk.gov.oiosi.security.lookup.CertificateLookupException;
import dk.gov.oiosi.security.lookup.ICertificateLookup;

/**
 * The ldap certificate lookup uses the ldap protocol to query for a given
 * certificate.
 * 
 * It implements the ICertificate lookup to garantee it has a speicific
 * interface for querying certificates.
 */
public class LdapCertificateLookup implements ICertificateLookup {

    /**
     * Local log.
     */
    private Log log = LogFactory.getLog(LdapCertificateLookup.class);

    /**
     * Settings.
     */
    private LdapSettings settings;

    /*
     * Certificate cache
     */
    private static ITimedCache<CertificateSubject, X509Certificate> certCache = new TimedCache<CertificateSubject, X509Certificate>(14 * 24 * 60 * 60 * 1000);
    
    /**
     * The LdapCertificateLookup constructor takes an ldap settings object as a
     * paramter. The setting object is used everytime a lookup is done in the
     * GetCertificate function.
     * 
     * @param s
     *            Setting object that contains the ldap settings to be used.
     */
    public LdapCertificateLookup(LdapSettings s) {
        settings = s;
    }

    /**
     * Default constructor. Attempts to read configuration settings from
     * configuration file
     */
    public LdapCertificateLookup() {
        this(ConfigurationHandler.getInstance().getLdapSettings());
    }

    /**
     * The implementation of the interface ICertificateLookups function.
     * 
     * It takes the subject string and queries the ldap server for a certificate
     * that satifies the condition in the subject string.
     * 
     * @param subject
     *            The subject string of an OCES certificate.
     * @return The certificate that satifies the conditions of the subject
     *         string.
     * @throws CertificateLookupException
     */
    public X509Certificate getCertificate(CertificateSubject subject) throws CertificateLookupException {
        if (subject == null)
            throw new ArgumentNullException("subjectSerialNumber");
        X509Certificate certificateToBeReturned = null;
        
        OutVariable<X509Certificate> value = new OutVariable<X509Certificate>();
        if (certCache.tryGetValue(subject, value)) {
        	return value.getVariable();
        }
    	StringBuffer ldapQueryString = new StringBuffer();
        
        // protocol and server
        ldapQueryString.append("ldap://");
        ldapQueryString.append( settings.getHost() );
        ldapQueryString.append(":");
        ldapQueryString.append(settings.getPort());

        // search url
        ldapQueryString.append("/c=dk??sub?(");
        ldapQueryString.append(subject.getPrefixedSerialNumber());
        ldapQueryString.append(")");

        log.debug("LDAP query is being executed...");
        log.debug("LDAP searchprefix: " + ldapQueryString);

        NamingEnumeration<SearchResult> answer;
        boolean hasmore;
        try {
            // Create the initial context
            DirContext ctx = new InitialLdapContext();

            // Perform the search by using the URL
            answer= ctx.search(ldapQueryString.toString(), null);
            // Close the context when we're done
            ctx.close();

            hasmore = answer.hasMore();
        } catch (CommunicationException e) {
            log.error("Could not perform search: " + e, e);
            throw new ConnectingToLdapServerFailedException(settings, e);
        } catch (NamingException e) {
            log.error("Could not create Ldap Search Context : " + e, e);
            throw new LdapCertificateLookupInitializationFailedException("Could not create Ldap Search Context", e);
        }
  
        if(!hasmore) {
            throw new LdapCertificateNotFoundException(subject);
        }

        byte[] certificateBytes;
        try {
            SearchResult sr = (SearchResult) answer.next();
            Attributes attrs = sr.getAttributes();
            Attribute attribute = (Attribute) attrs.get("userCertificate;binary");

            if (attribute == null) {
            	throw new ConvertingSearchResultToCertificateFailedException("Could not get certificate attribute from Ldap Search: " + "userCertificate;binary");
            }
            
            certificateBytes = (byte[]) attribute.get();
        } catch(NamingException e) {
            log.error("Could not get certificate attribute from Ldap Search : " + e, e);
            throw new ConvertingSearchResultToCertificateFailedException("Could not get certificate attribute from Ldap Search", e);            
        }

        try {
            CertificateFactory cf = CertificateFactory.getInstance("X.509");
            Collection<?> c = cf.generateCertificates(new ByteArrayInputStream(certificateBytes));
            certificateToBeReturned = (X509Certificate) c.iterator().next();
        } catch (CertificateException e) {
            log.error("Could not create certificate form ldap result : " + e, e);
            throw new ConvertingSearchResultToCertificateFailedException("Could not create certificate form ldap result", e);            
        }

        if (certificateToBeReturned == null) {
            log.info("Certificate not found with LDAP query:" + subject.getSerialNumberValue());
            throw new LdapCertificateNotFoundException(subject);
        }
        
        try {
            hasmore = answer.hasMore();
        } catch (NamingException e) {
            log.error("Could not create Ldap Search Context : " + e, e);
            throw new LdapCertificateLookupInitializationFailedException("Could not create Ldap Search Context", e);
        }        
        if(hasmore) {
            throw new LdapMultipleCertificatesFoundException(subject);
        }

        certCache.set(subject, certificateToBeReturned);
        log.debug("LDAP query done...");
        return certificateToBeReturned;
    }
}
