/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and 
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis Søgaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */

package dk.gov.oiosi.security.revocation.crl;

import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.security.cert.CRLException;
import java.security.cert.Certificate;
import java.security.cert.CertificateFactory;
import java.security.cert.X509CRL;
import java.util.Date;

import dk.gov.oiosi.security.revocation.CheckCertificateRevokedUnexpectedException;

/**
 * Class used for storing CRLs retrieved from URL's in X509 certificates
 */
class CrlInstance
{
	private CertificateFactory cf;
	private X509CRL data;
	private URL url;

	/**
	 * Creates a new CRLInstance instance.
	 * 
	 * @param cf A factory used for generating X509CRL class's from a stream.
	 * @param url The URL corresponding to the CRL.
	 */
	public CrlInstance(CertificateFactory cf, URL url)
	{
		this.cf = cf;
		this.data = null;
		this.url = url;
	}
	   
	/**
	 * Checks whether the given certificate cert is revoked.
	 * 
	 * @param cert The certificate to check whether revoked or not
	 * @return Returns true if revoked, false otherwise.
	 * @throws CheckCertificateRevokedUnexpectedException
	 */
	public synchronized boolean isRevoked(Certificate cert) throws CheckCertificateRevokedUnexpectedException
	{
		if (!cacheValid())
		{
			upgradeData();
		}
		
		// Reads the data and unlocks.
		boolean isRevoked = data.isRevoked(cert);
		return isRevoked;
	}
	   
	/**
	 * Downloads a new CRL from the URL stored in the class.
	 * 
	 * @throws CheckCertificateRevokedUnexpectedException
	 */
	private void upgradeData() throws CheckCertificateRevokedUnexpectedException
	{
		try 
		{
			HttpURLConnection urlConnection = (HttpURLConnection)url.openConnection();
			
			// Only download .crl file if it is present on the server.
			if(urlConnection.getResponseCode() == HttpURLConnection.HTTP_OK)
			{
				InputStream input = urlConnection.getInputStream();
				
				// Downloads the .crl file into an X509CRL object.
				data = (X509CRL)cf.generateCRL(input);
				input.close();
				
				return; // Everything went well.
			}
			
			throw new CheckCertificateRevokedUnexpectedException(new Exception("CRL could not be downloaded: " + urlConnection.getResponseMessage()));
		} catch (IOException e) {
			// Could not download new crl
			throw new CheckCertificateRevokedUnexpectedException(e);
		} catch (CRLException e) {
			// Something was wrong with crl
			throw new CheckCertificateRevokedUnexpectedException(e);
		}
	}
	   
	/**
	 * Checks whether the CRL is still valid.
	 * 
	 * @return Returns true if CRL is still valid, false otherwise.
	 */
	private boolean cacheValid()
	{
		return data != null && data.getNextUpdate().before(new Date(System.currentTimeMillis()));
	}

	/**
	 * The NextUpdate value from the CRL
	 * 
	 * @return The NextUpdate value from the CRL
	 */
	public synchronized Date getNextUpdate() 
	{
		Date date = data != null ? data.getNextUpdate() : new Date(0);
		return date;
	}
}
