/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis Søgaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *   Christian Uldall Pedersen (christian.u.pedersen@accenture.com)
 *
 */
package dk.gov.oiosi.security.revocation.crl;

import java.io.IOException;
import java.net.URL;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.LinkedList;
import java.util.List;

import org.bouncycastle.asn1.DERString;
import org.bouncycastle.asn1.x509.CRLDistPoint;
import org.bouncycastle.asn1.x509.DistributionPoint;
import org.bouncycastle.asn1.x509.GeneralName;
import org.bouncycastle.asn1.x509.GeneralNames;
import org.bouncycastle.asn1.x509.X509Extensions;
import org.bouncycastle.x509.extension.X509ExtensionUtil;

import dk.gov.oiosi.security.revocation.CheckCertificateRevokedUnexpectedException;
import dk.gov.oiosi.security.revocation.IRevocationLookup;
import dk.gov.oiosi.security.revocation.RevocationException;
import dk.gov.oiosi.security.revocation.RevocationResponse;

/**
 * Class for checking certificate revocation status against a CRL.
 * 
 * Makes use of a cache for storing CRLs
 */
public class CrlLookup implements IRevocationLookup
{
	/**
	 * Cache used for storing CRLs
	 */
	private static CrlCache cache;
	
	static 
	{
		try 
		{
			cache = new CrlCache();
		} catch (CertificateException e)
		{
			throw new RuntimeException(e.getMessage());
		}
	}
	
	/**
	 * Checks a certificate status in a CRL.
	 * 
	 * @param certificate
	 *            The certificate to check
	 * @return The RevocationResponse object that contains the result
	 * @throws CertificateRevokedException
	 */
	@Override
	public RevocationResponse checkCertificate(X509Certificate certificate) throws RevocationException
	{
		/*
		 * Assumptions:
		 * - Certificate has an CRL Distribution Points extension value.
		 * - An HTTP distribution point is present.
		 */

		RevocationResponse response = new RevocationResponse();
		
		RevocationException innerException = null;
		
    	if (certificate != null)
    	{
    		// Retrieve URL distribution points
    		List<URL> URLs = getURLs(certificate);
    		
    		for (URL url : URLs)
    		{
    			CrlInstance crl = cache.getCRL(url);
    			
    			try 
    			{
					if(!crl.isRevoked(certificate))
					{
						response.setIsValid(true);
						response.setNextUpdate(crl.getNextUpdate());
						return response;
					}
					else
					{
						response.setIsValid(false);
						return response;
					}
				} catch (CheckCertificateRevokedUnexpectedException e) 
				{
					// There was an error in checking the certificate. Try the next url.
					innerException = e;
				}
    		}
        }

    	// If any errors happen during CRL check, then abort the processing of the message.
    	if (innerException != null) {
    		throw innerException;
    	} else {
            throw new RevocationException("Fault happened during CRL lookup");
    	}
        
	}
	
	/**
	 * Gets a list of URLs from the specified certificate.
	 * 
	 * @param cert The certificate to find the URLs in.
	 * @return A list of CRL URLs in the certificate
	 */
	private List<URL> getURLs(X509Certificate cert)
	{
		List<URL> urls = new LinkedList<URL>(); 
		
		// Retrieves the raw ASN1 data of the CRL Dist Points X509 extension
		byte[] cdp = cert.getExtensionValue(X509Extensions.CRLDistributionPoints.getId());
		if (cdp != null)
        {
			try 
			{
				// Wraps the raw data in a container class
				CRLDistPoint crldp = CRLDistPoint.getInstance(X509ExtensionUtil.fromExtensionValue(cdp));
			
				DistributionPoint[] distPoints = crldp.getDistributionPoints();
	    		
	    		for (DistributionPoint dp : distPoints)
	    		{
	    			// Only use the "General name" data in the distribution point entry.
	    			GeneralNames gns = (GeneralNames) dp.getDistributionPoint().getName();

	    			for (GeneralName name : gns.getNames())
	    			{
	    				// Only retrieve URLs
	    				if (name.getTagNo() == GeneralName.uniformResourceIdentifier)
	    				{
	    					DERString s = (DERString) name.getName();
	    					urls.add(new URL(s.getString()));
	    				}
	    			}
	    		}
			} catch (IOException e) {
				// Could not retrieve the CRLDistPoint object. Just return empty url list.
			}
        }
		
		return urls;
	}
}
