/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis Søgaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */

package dk.gov.oiosi.xml;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.net.URL;

import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamReader;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMXMLParserWrapper;
import org.apache.axiom.om.impl.llom.factory.OMXMLBuilderFactory;
import org.apache.axis2.util.XMLUtils;
import org.apache.commons.configuration.ConfigurationUtils;
import org.w3c.dom.Element;

import dk.gov.oiosi.exception.MainException;

/**
 * Wraps the Java standard w3 and Axis Axiom XML-document representation.
 */
public class XmlDocument {

    protected String fileName = null;

    /**
     * org.w3c.dom representation of the document.
     */
    private Element w3Document;

    /**
     * Axiom representation of the document.
     */
    private OMElement omDocument;

    /**
     * Default constructor.
     */
    public XmlDocument() {}

    /**
     * Constructor that takes a w3c document.
     * 
     * @param doc
     *            w3c document
     */
    public XmlDocument(Element doc) {
        setDocument(doc);
    }

    /**
     * Constructor that takes an Axiom document.
     * 
     * @param doc
     *            Axiom document
     */
    public XmlDocument(OMElement doc) {
        setDocument(doc);
    }
    
    /**
     * Set document as an w3c document
     * 
     * @param doc
     *            w3c document
     */
    public void setDocument(Element doc) {
        w3Document = doc;
        omDocument = null;
    }

    /**
     * Set document as Axiom document
     * 
     * @param doc
     *            Axiom document
     */
    public void setDocument(OMElement doc) {
        w3Document = null;
        omDocument = doc;
    }

    /**
     * Returns the document as w3c document. Convert if needed.
     * 
     * @return w3c document
     * @throws XmlDocumentException
     */
    public Element getW3Document() throws XmlDocumentException {
        if (omDocument == null && w3Document == null) {
            return null;
        }
        if (w3Document != null) {
            return w3Document;
        }
        try {
            return XMLUtils.toDOM(omDocument);
        } catch (Exception e) {
            throw new XmlDocumentException(omDocument);
        }
    }

    /**
     * Returns the document as Axiom document. Convert if needed.
     * 
     * @return Axiom document
     * @throws XmlDocumentException
     */
    public OMElement getOMDocument() throws XmlDocumentException {
        if (omDocument == null && w3Document == null) {
            return null;
        }
        if (omDocument != null) {
            return omDocument;
        }
        try {
            return XMLUtils.toOM(w3Document);
        } catch (Exception e) {
            throw new XmlDocumentException(w3Document);
        }
    }

    /**
     * Loads the document from file system. Axiom format is the default.
     * 
     * @param file
     *            Name of document file
     * @throws MainException
     * @throws FileNotFoundException
     */
    public void load(String file) throws MainException, FileNotFoundException {
        XMLStreamReader parser;
        try {

            // Try to read Xml file. Try reading as resource - fallback to reading as file.
            try {
                //URL url = XmlDocument.class.getClassLoader().getResource(file);
            	URL url = ConfigurationUtils.locate(file);
            	
                parser = XMLInputFactory.newInstance().createXMLStreamReader(url.openStream());
                fileName = url.toString();
            } catch (Exception e) {
                parser = XMLInputFactory.newInstance().createXMLStreamReader(new FileInputStream(file));
                fileName = file;
            }
            OMXMLParserWrapper builder = OMXMLBuilderFactory.createStAXOMBuilder(OMAbstractFactory.getOMFactory(), parser);
            OMElement doc = builder.getDocumentElement();
            doc.build();

            omDocument = doc;
        } catch (FileNotFoundException e) {
            throw e;
        } catch (Exception e) {
            throw new XmlException("Failed to read XML : " + e, e);
        }
    }
}
