/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis Søgaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */

package dk.gov.oiosi.xml.schema;

import java.net.URL;

import javax.xml.XMLConstants;
import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Element;

import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.XmlSchema;

/**
 * Represents a validator that can validate whether an XML-document is valid for a given XML-schema.
 */
public class SchemaValidator {

    /**
     * local log.
     */
    private static Log log = LogFactory.getLog(SchemaValidator.class);

    /**
     * Validates whether the XML-document is valid to the given XML-schema. Further it ensures that the schema and the document has
     * the same namespace. If either the document is invalid or the namespaces are incorrect then an exception is thrown.
     * 
     * @param xmlDocument
     *            document to check
     * @param xmlSchema
     *            schema for validating
     * @throws SchemaValidationFailedException
     */
    public void schemaValidateXmlDocument(XmlDocument xmlDocument, XmlSchema xmlSchema) throws SchemaValidationException {
        try {

            Element w3doc = xmlDocument.getW3Document();

            // create and load Schema
            SchemaFactory factory = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
            Schema schema;
            
            // Try to read schema file. Try reading as resource - fallback to reading as file.
            try {
                URL url = SchemaValidator.class.getClassLoader().getResource(xmlSchema.getFileName());
                schema = factory.newSchema(url);
            } catch (Exception e) {
                Source source = new StreamSource(xmlSchema.getFileName());
                schema = factory.newSchema(source);
            }
            
            // Java Schema validator validates namespaces automatically
            // checkNamespace(xmlDocument, xmlSchema);
            
            // create a validator
            Validator validator = schema.newValidator();

            // validate document
            validator.validate(new DOMSource(w3doc));
        } catch (Exception e) {
            log.error("Schema check failed : " + e);
            throw new SchemaValidationFailedException(xmlDocument, e);
        }
    }
}
