/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and 
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis Søgaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */

package dk.gov.oiosi.xml.schematron;

import javax.xml.transform.Transformer;

import dk.gov.oiosi.common.OutVariable;
import dk.gov.oiosi.common.cache.QuantityCache;
import dk.gov.oiosi.exception.ArgumentNullException;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.xslt.XsltUtility;

public class SchematronStore {
    private Object _lockObject = new Object();
    private QuantityCache<String, Transformer> _cache;

    /**
     * Constructor that uses a default of max two coherent compiled stylesheeets
     * in memory.
     */
    public SchematronStore() {
          _cache = new QuantityCache<String, Transformer>(2);
    }

    /**
     * Constructor that takes the configuration for the schematron store
     */
    public SchematronStore(ISchematronStoreConfig configuration) {
        if (configuration == null) throw new ArgumentNullException("configuration");
        InitFromConfiguration(configuration);
    }

    /**
     * Gets the compiled schematron stylesheet from a given path.
     * @param schematronPath
     */
    public Transformer getCompiledSchematron(String schematronPath) throws FailedToLoadSchematronStylesheetException {
        if (schematronPath == null) throw new ArgumentNullException("path");
        Transformer compiledSchematron = null;
        synchronized (_lockObject) {
        	OutVariable<Transformer> result = new OutVariable<Transformer>(); 
            if (_cache.tryGetValue(schematronPath, result)) return result.getVariable();
            compiledSchematron = loadCompiledSchematron(schematronPath);
            _cache.add(schematronPath, compiledSchematron);
            return compiledSchematron;
        }
    }

    private void InitFromConfiguration(ISchematronStoreConfig config) {
        int maxStylesheetsInMemory = (int)config.getMaxCompiledStylesheetsInMemory();
        _cache = new QuantityCache<String, Transformer>(maxStylesheetsInMemory);
    }

    private Transformer loadCompiledSchematron(String path) throws FailedToLoadSchematronStylesheetException {
        XmlDocument xmlStylesheet = new XmlDocument();
        XsltUtility xsltUtility = new XsltUtility();
        try {
            xmlStylesheet.load(path);
            return xsltUtility.precompiledStyleSheet(xmlStylesheet);
        }
        catch (Exception ex) {
            throw new FailedToLoadSchematronStylesheetException(path, ex);
        }
    }
}
