/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis Søgaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */

package dk.gov.oiosi.xml.schematron;

import dk.gov.oiosi.exception.ArgumentNullException;
import dk.gov.oiosi.xml.XmlDocument;

/**
 * Configuration to the schematron validation.
 */
public class SchematronValidationConfig {

    /**
     * Path to schematron document.
     */
    private String schematronDocumentPath = "";

    /**
     * XPath to find error
     */
    private String errorXPath = "";

    /**
     * XPath to find error message.
     */
    private String errorMessageXPath = "";

    /**
     * Schematron document.
     */
    private XmlDocument schematronDocument;

    /**
     * Default constructor, should not be used.
     */
    public SchematronValidationConfig() {
    }

    /**
     * Constructor that takes the path to the schematron document, the error
     * XPath and the error message XPath.
     * 
     * @param schematronDocumentPath
     *            The schematron document path.
     * @param errorXPath
     *            The error XPath
     * @param errorMessageXPath
     *            The error message XPath
     */
    public SchematronValidationConfig(String schematronDocumentPath, String errorXPath, String errorMessageXPath) {
        if (schematronDocumentPath == null)
            throw new ArgumentNullException("schematronDocumentPath");
        if (errorXPath == null)
            throw new ArgumentNullException("errorXPath");
        if (errorMessageXPath == null)
            throw new ArgumentNullException("errorMessageXPath");
        this.schematronDocumentPath = schematronDocumentPath;
        this.errorXPath = errorXPath;
        this.errorMessageXPath = errorMessageXPath;
    }

    /**
     * Gets and sets the schematron document path.
     * 
     * @return path to schematron document
     */
    public String getSchematronDocumentPath() {
        return schematronDocumentPath;
    }

    /**
     * Gets and sets the schematron document path.
     * 
     * @param value
     *            to schematron document
     */
    public void setSchematronDocumentPath(String value) {
        if (value == null)
            throw new ArgumentNullException("SchematronDocumentPath.value");

        schematronDocumentPath = value;
    }

    /**
     * Gets and sets the XPath expression that gets the error results from the
     * schematron result.
     * 
     * @return XPath to error
     */
    public String getErrorXPath() {
        return errorXPath;
    }

    /**
     * Gets and sets the XPath expression that gets the error results from the
     * schematron result.
     * 
     * @param value
     *            to error
     */
    public void setErrorXPath(String value) {
        if (value == null)
            throw new ArgumentNullException("ErrorXPath.value");

        errorXPath = value;
    }

    /**
     * Gets and sets the XPath expression that gets the error messages from the
     * schematron result.
     * 
     * @return XPath to error message
     */
    public String getErrorMessageXPath() {
        return errorMessageXPath;
    }

    /**
     * Gets and sets the XPath expression that gets the error messages from the
     * schematron result.
     * 
     * @param value
     *            to error message
     */
    public void setErrorMessageXPath(String value) {
        if (value == null)
            throw new ArgumentNullException("ErrorMessageXPath.value");

        errorMessageXPath = value;
    }

    /**
     * Gets the schematron document. The first time this method is called then
     * the XML-document will be loaded from disc into memory.
     * 
     * @return XML-document holding stylesheet
     */
    public XmlDocument getSchematronDocument() throws SchematronErrorException {
        synchronized (this) {
            if (schematronDocument == null)
                loadSchematronDocument();
        }
        return schematronDocument;
    }

    /**
     * Load the  schematron stylesheet from file
     * 
     * @throws SchematronErrorException
     */
    private void loadSchematronDocument() throws SchematronErrorException {
        try {

            XmlDocument xmlDocument = new XmlDocument();
            xmlDocument.load(schematronDocumentPath);
            schematronDocument = xmlDocument;
        } catch (Exception ex) {
            throw new SchematronErrorException("Failed to load schematron document" + ex);
        }
    }

}
