/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis Søgaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */

package dk.gov.oiosi.xml.xslt;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;

import javax.xml.stream.XMLStreamException;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.URIResolver;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Element;

import dk.gov.oiosi.common.RaspLibraryException;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.XmlDocumentException;
import dk.gov.oiosi.xml.XmlException;

/**
 * This utility is used for xslt transforming documents.
 */
public class XsltUtility {

    /**
     * logger.
     */
    private static Log log = LogFactory.getLog(XsltUtility.class);

    /**
     * Method to transform any xmldocument with a given xslt.
     *
     * @param xmlDoc The xml document to transform.
     * @param stylesheet The xslt to transform xml document.
     * @return The transformed xml document
     * @throws XmlException Xml transformations errors
     * @throws Exception yet unknown errors
     */
    public XmlDocument transformXML(XmlDocument xmlDoc, XmlDocument stylesheet) throws XmlException {
        try {
            Transformer trans = precompiledStyleSheet(stylesheet);
            return transformXml(xmlDoc, trans);
        } catch (TransformerConfigurationException e) {
            log.error("XsltUtility.transformXML Failed : " + e, e);
            throw new XmlException("XSL transformation error: " + e.getMessage(), e);
        } catch (TransformerException e) {
            log.error("XsltUtility.transformXML Failed : " + e, e);
            throw new XmlException("XSL transformation error: " + e.getMessage(), e);
        } catch (XMLStreamException e) {
            log.error("XsltUtility.transformXML Failed : " + e, e);
            throw new XmlException("XSL transformation error: " + e.getMessage(), e);
        } catch (Exception e) {
            log.error("XsltUtility.transformXML Failed : " + e, e);
            throw new RaspLibraryException("RASP transformation error ! " + e, e);
        }
    }
    
    /**
     * Method that returns the precompiled XSLT stylesheet from the given XML document.
     */
    public Transformer precompiledStyleSheet(XmlDocument stylesheet) throws XmlDocumentException, TransformerConfigurationException {
    	Source xsltSource = new DOMSource(stylesheet.getW3Document().getOwnerDocument());
    	TransformerFactory transFact = TransformerFactory.newInstance();

        return transFact.newTransformer(xsltSource);
    }
    
    /**
     * Method that transforms the XML document from a precompiled XSLT stylesheet
     */
    public XmlDocument transformXml(XmlDocument document, Transformer transform) throws XmlDocumentException, TransformerException, XMLStreamException {
        Element w3doc = document.getW3Document();
        StreamResult sr = new StreamResult(new ByteArrayOutputStream());
        log.debug("Doing transformation...");
        transform.transform(new DOMSource(w3doc.getOwnerDocument()), sr);

        String result = sr.getOutputStream().toString();
        log.debug("Creating AXOMbuilder from: " + result);
        StAXOMBuilder builder = new StAXOMBuilder(new ByteArrayInputStream(result.getBytes()));
        return new XmlDocument(builder.getDocumentElement());
    }
}
