/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis S�gaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.test.integration.security.ldap;

import java.security.cert.CertificateNotYetValidException;
import java.security.cert.X509Certificate;

import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.security.CertificateSubject;
import dk.gov.oiosi.security.ldap.ConnectingToLdapServerFailedException;
import dk.gov.oiosi.security.ldap.LdapCertificateLookup;
import dk.gov.oiosi.security.ldap.LdapCertificateNotFoundException;
import dk.gov.oiosi.security.ldap.LdapLookupFactory;
import dk.gov.oiosi.security.ldap.LdapSettings;
import dk.gov.oiosi.security.lookup.CertificateLookupException;
import dk.gov.oiosi.security.lookup.CertificateNotFoundException;
import dk.gov.oiosi.security.lookup.ICertificateLookup;
import dk.gov.oiosi.security.lookup.MultipleCertificatesFoundException;
import dk.gov.oiosi.security.lookup.SearchFailedException;
import dk.gov.oiosi.security.validation.CertificateExpiredException;
import dk.gov.oiosi.security.validation.CertificateNotActiveException;
import dk.gov.oiosi.security.validation.CertificateValidator;
import dk.gov.oiosi.test.TestConstants;
import dk.gov.oiosi.test.stubs.LdapCertificateLookupStub;

import junit.framework.TestCase;

/**
 * jUnit test class for getting certificate from LDAP server.
 */
public class LdapRequestTest extends TestCase {

    static {
        ConfigurationHandler.setConfigurationFile(TestConstants.PATH_TEST_RASPCONFIGURATION);
    }

    /**
     * Subject DN, itst company certificate.
     */
    private static final String UNKNOWN_SUBJECTDN = "SERIALNUMBER=CVR:11111111-UID:1172691221366 + CN=IT- og Telestyrelsen - OIOSI test virksomhedssignatur, O=IT- og Telestyrelsen // CVR:26769388, C=DK";

    /**
     * Test getting certerficate. Configuration comes from the configuration
     * file.
     * 
     * @param subjectString
     *            subject serial number
     * @throws CertificateLookupException 
     * @throws SearchFailedException 
     * @throws MultipleCertificatesFoundException 
     * @throws CertificateNotFoundException 
     * @throws CertificateExpiredException 
     * @throws CertificateNotActiveException 
     */
    private void performQuery(String subjectString) throws CertificateNotFoundException, MultipleCertificatesFoundException, SearchFailedException, CertificateLookupException, CertificateNotActiveException, CertificateExpiredException {
        ICertificateLookup ldapLookup = LdapLookupFactory.createLdapLookupClient();
        CertificateSubject certSubject = new CertificateSubject(subjectString);
        X509Certificate cert = ldapLookup.getCertificate(certSubject);
        CertificateValidator.validateCertificate(cert);
    }


    public void testWrongLdapServer() {
        LdapSettings setting = new LdapSettings();
        setting.setHost("localhost");
        LdapCertificateLookup ldapLookup = new LdapCertificateLookup(setting);
        try {
            CertificateSubject certSubject = new CertificateSubject(TestConstants.TEST_CERTIFICATE_SUBJECT_COMPANY);
            ldapLookup.getCertificate(certSubject);
            fail("This test should fail");
        } catch (ConnectingToLdapServerFailedException e) {
            assertTrue(true);
        } catch (CertificateLookupException e) {
            fail("Wrong Exception" + e);
        }
    }
    
    /**
     * Get device certificate.
     */
    public void testFunction() {
        try {
            performQuery(TestConstants.TEST_CERTIFICATE_SUBJECT_FUNCTION);
        } catch(Exception e) {
            fail("This test should not fail : " + e); 
        }
    }

    /**
     * Get company certificate.
     */
    public void testVirksomhed() {
        try { 
            performQuery(TestConstants.TEST_CERTIFICATE_SUBJECT_COMPANY);
        } catch(Exception e) {
            fail("This test should not fail : " + e); 
        }
    }

    /**
     * Get company certificate.
     */
    public void testUnknowSubject() {
        try {
            performQuery(UNKNOWN_SUBJECTDN);
            fail("This test should fail : " + UNKNOWN_SUBJECTDN ); 
        } catch(LdapCertificateNotFoundException e) {
            assertTrue(true);
        } catch (Exception e) {
            fail("Wrong Exception" + e);
        }
    }
}
