/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis S�gaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */

package dk.gov.oiosi.test.integration.security.revocation;

import java.io.FileInputStream;
import java.security.KeyStore;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import junit.framework.TestCase;
import dk.gov.oiosi.common.TimeSpan;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.security.revocation.IRevocationLookup;
import dk.gov.oiosi.security.revocation.RevocationResponse;
import dk.gov.oiosi.security.revocation.RevocationException;
import dk.gov.oiosi.security.revocation.crl.CrlLookup;
import dk.gov.oiosi.test.TestConstants;

/**
 * jUnit test class for testing certificate revocation against crl.
 */
public class CrlRequestTest extends TestCase {


	static {
		ConfigurationHandler
				.setConfigurationFile("./conf/RaspConfigurationTestServers.xml");
	}

	/**
	 * Read certificate from keystore.
	 * 
	 * @param keylabel
	 *            keylabel
	 * @return certificate
	 * @throws Exception
	 */
	protected X509Certificate getCertificate(String keylabel) throws Exception {
		// define receiver certificate
		KeyStore ks = KeyStore.getInstance("JKS");
		ks.load(new FileInputStream(TestConstants.PATH_KEYSTORE_TEST),
				TestConstants.PASSWORD_KEYSTORE_TEST.toCharArray());

		X509Certificate client = (X509Certificate) ks.getCertificate(keylabel);
		return client;
	}

	/**
	 * Test crl for revoked certificate.
	 */
	public void testRevoked()
	{
		try 
		{
			X509Certificate cert = getCertificate("server_tdc_revoked");
			
			IRevocationLookup crlLookup = new CrlLookup();
			RevocationResponse response = crlLookup.checkCertificate(cert);

			assertFalse(response.isValid());
		} catch (Exception e) {
			fail("Unknow failure " + e);
		}
	}
	
	/**
	 * Test crl for non-revoked certificate.
	 */
	public void testNonRevoked()
	{
		try 
		{
			X509Certificate cert = getCertificate("server_tdc");
			
			IRevocationLookup crlLookup = new CrlLookup();
			RevocationResponse response = crlLookup.checkCertificate(cert);

			assertTrue(response.isValid());
		} catch (Exception e) {
			fail("Unknow failure " + e);
		}
	}

	public void testExpiredButNotRevoked() throws RevocationException
	{
		try 
		{
			X509Certificate cert = getCertificate("server_tdc_expired");
			
			IRevocationLookup crlLookup = new CrlLookup();
			RevocationResponse response = crlLookup.checkCertificate(cert);

			assertTrue(response.isValid());
		} catch (Exception e) {
			fail("Unknow failure " + e);
		}
	}

	
	public void testLookTestMultiThread() throws InterruptedException {
		List<Thread> threads = new ArrayList<Thread>();
		for (int i = 0; i < 32; i++) {
			Runnable runnable = new ThreadCertificateCheck();
			Thread thread = new Thread(runnable);
			thread.start();
			threads.add(thread);
		}

		boolean run = true;
		while (run) {
			boolean anyAlive = false;
			for(Thread thread : threads) {
				anyAlive = thread.isAlive();
				if (anyAlive) break;
			}
			if (anyAlive) {
				Thread.sleep(1000);
				continue;
			}
			run = false;
		}
	}

	class ThreadCertificateCheck implements Runnable {
		public void run() {
			try {
	         CrlLookup crlLookup = new CrlLookup();
	         for (int i = 0; i < 16; i++) {
	             Random random = new Random();
	             int select = random.nextInt(2);
	             if (select < 1) {
	            	 System.out.println("{0} ThreadCertificateCheck number:{1} certificate:0");
	            	 X509Certificate cert = getCertificate("server_tdc");
	                 RevocationResponse response = crlLookup.checkCertificate(cert);
	                 assertTrue(response.isValid());
	             }
	             else {
	            	 System.out.println("{0} ThreadCertificateCheck number:{1} certificate:1");
	            	 X509Certificate cert = getCertificate("server_tdc_revoked");
	                 RevocationResponse response = crlLookup.checkCertificate(cert);
	                 assertFalse(response.isValid());
	             }
	             System.out.println("{0} ThreadCertificateCheck number:{1} done");
	         }
			} 
			catch (Exception ex) {
				ex.printStackTrace();
			}
	     }
	}
	

	
	/**
	 * psvm for testing.
	 * 
	 * @param args
	 *            args
	 */
	public static void main(String[] args) {
		CrlRequestTest test = new CrlRequestTest();
		test.testRevoked();

	}
}
