/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis Soegaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */

package dk.gov.oiosi.test.integration.security.revocation;

import java.io.FileInputStream;
import java.io.InputStream;
import java.security.KeyStore;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;

import junit.framework.Assert;
import junit.framework.TestCase;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.security.revocation.IRevocationLookup;
import dk.gov.oiosi.security.revocation.RevocationException;
import dk.gov.oiosi.security.revocation.RevocationResponse;
import dk.gov.oiosi.security.revocation.ocsp.OcspConfig;
import dk.gov.oiosi.security.revocation.ocsp.OcspLookup;
import dk.gov.oiosi.test.TestConstants;

/**
 * jUnit test class for testing certificate against ocsp server.
 */
public class OcspRequestTest extends TestCase {

	private Log log = LogFactory.getLog(OcspRequestTest.class);

	static {
        ConfigurationHandler.setConfigurationFile(TestConstants.PATH_TEST_RASPCONFIGURATION);
    }

	/**
	 * Read certificate from keystore.
	 *
	 * @param keylabel keylabel
	 * @return certificate
	 * @throws Exception
	 */
	protected X509Certificate getCertificate(String keylabel) throws Exception {
		// define receiver certificate
		KeyStore ks = KeyStore.getInstance("JKS");
		ks.load(new FileInputStream(TestConstants.PATH_KEYSTORE_TEST),
				TestConstants.PASSWORD_KEYSTORE_TEST.toCharArray());

		X509Certificate client = (X509Certificate) ks.getCertificate(keylabel);
		return client;
	}

	/**
	 * Test crl for revoked certificate.
	 */
	public void testRevoked()
	{
		try
		{
			X509Certificate cert = getCertificate("server_tdc_revoked");
			IRevocationLookup crlLookup = new OcspLookup();
			RevocationResponse response = crlLookup.checkCertificate(cert);
			assertFalse(response.isValid());
		} catch (Exception e) {
			fail("Unknow failure " + e);
		}
	}

	/**
	 * Test crl for non-revoked certificate.
	 */
	public void testNonRevoked()
	{
		try
		{
			X509Certificate cert = getCertificate("server_tdc");

			IRevocationLookup crlLookup = new OcspLookup();
			RevocationResponse response = crlLookup.checkCertificate(cert);

			assertTrue(response.isValid());
		} catch (Exception e) {
			fail("Unknow failure " + e);
		}
	}

	public void testExpiredButNotRevoked() throws RevocationException
	{
		try
		{
			X509Certificate cert = getCertificate("server_tdc_expired");

			IRevocationLookup crlLookup = new OcspLookup();
			RevocationResponse response = crlLookup.checkCertificate(cert);

			assertTrue(response.isValid());
		} catch (Exception e) {
			fail("Unknow failure " + e);
		}
	}

	public void testNonRevokedWithOcspServerFromCertificate() {
		try {
			OcspConfig ocspConfig = new OcspConfig();
			ocspConfig.setDefaultTimeoutMsec(10000);
			OcspLookup ocspLookup = new OcspLookup(ocspConfig);

			X509Certificate cert = getCertificate("server_tdc");
			RevocationResponse response = ocspLookup.checkCertificate(cert);
			Assert.assertTrue(response.isValid());
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	public void testRevokedWithOcspServerFromCertificate() {
		try {
			OcspConfig ocspConfig = new OcspConfig();
			ocspConfig.setDefaultTimeoutMsec(10000);
			OcspLookup ocspLookup = new OcspLookup(ocspConfig);

			X509Certificate cert = getCertificate("server_tdc_revoked");
			RevocationResponse response = ocspLookup.checkCertificate(cert);
			assertFalse("This certificate should have been catched as revoked", response.isValid());
		}
		catch (Exception e) {
			fail(e.getMessage());
		}

	}

	public void testNonRevokedWithOcspServerFromConfig() {
		try {
			OcspConfig ocspConfig = new OcspConfig();
			ocspConfig.setDefaultTimeoutMsec(10000);
			ocspConfig.setServerUrl("http://test.ocsp.certifikat.dk/ocsp/status");
			OcspLookup ocspLookup = new OcspLookup(ocspConfig);
			X509Certificate cert = getCertificate("server_tdc");
			RevocationResponse response = ocspLookup.checkCertificate(cert);
			Assert.assertTrue(response.isValid());
		}
		catch (Exception e) {
			fail(e.getMessage());
		}
	}

	public void testRevokedWithOcspServerFromConfig() {
		try {
			OcspConfig ocspConfig = new OcspConfig();
			ocspConfig.setDefaultTimeoutMsec(10000);
			ocspConfig.setServerUrl("http://test.ocsp.certifikat.dk/ocsp/status");
			OcspLookup ocspLookup = new OcspLookup(ocspConfig);
			X509Certificate cert = getCertificate("server_tdc_revoked");
			RevocationResponse response = ocspLookup.checkCertificate(cert);
			assertFalse("This certificate should have been catched as revoked", response.isValid());
		}
		catch (Exception e) {
			fail(e.getMessage());
		}
	}


	public void testGetServerUriFromCertificateWithoutRootCert() {

		try {
			OcspConfig ocspConfig = new OcspConfig();
			ocspConfig.setDefaultTimeoutMsec(10000);

			OcspLookup ocspLookup = new OcspLookup(ocspConfig);

			X509Certificate cert;

			cert = getCertificate("client_person");
			log.info(ocspLookup.GetServerUriFromCertificate(cert));
			Assert.assertEquals("http://test.ocsp.certifikat.dk/ocsp/status",
					ocspLookup.GetServerUriFromCertificate(cert));
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	public void testGetServerUriFromCertificateWithRootCert() {
		try {
			OcspConfig ocspConfig = new OcspConfig();
			ocspConfig.setDefaultTimeoutMsec(10000);
			InputStream inStream = new FileInputStream(TestConstants.PATH_CERTIFICATE_ROOT);
			CertificateFactory cf = CertificateFactory.getInstance("X.509");
			X509Certificate rootcert = (X509Certificate) cf.generateCertificate(inStream);

			OcspLookup ocspLookup = new OcspLookup(ocspConfig, rootcert);

			X509Certificate cert = getCertificate("client_person");
			log.info(ocspLookup.GetServerUriFromCertificate(cert));
			Assert.assertEquals("http://test.ocsp.certifikat.dk/ocsp/status",
					ocspLookup.GetServerUriFromCertificate(cert));
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	/**
	 * psvm for testing.
	 *
	 * @param args
	 *            args
	 */
	public static void main(String[] args) {
		OcspRequestTest test = new OcspRequestTest();
		test.testRevoked();

	}
}