package dk.gov.oiosi.test.integration.uddi;

import java.net.URISyntaxException;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.LinkedList;
import java.util.List;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import dk.gov.oiosi.addressing.Identifier;
import dk.gov.oiosi.addressing.IdentifierDuns;
import dk.gov.oiosi.addressing.IdentifierEan;
import dk.gov.oiosi.common.Version;
import dk.gov.oiosi.communication.Uri;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.test.TestConstants;
import dk.gov.oiosi.uddi.EndpointAddressTypeCode;
import dk.gov.oiosi.uddi.LookupParameters;
import dk.gov.oiosi.uddi.UddiConfig;
import dk.gov.oiosi.uddi.UddiGuidId;
import dk.gov.oiosi.uddi.UddiId;
import dk.gov.oiosi.uddi.UddiLookupClient;
import dk.gov.oiosi.uddi.UddiLookupResponse;
import dk.gov.oiosi.uddi.ProcessRoleDefinition;
import dk.itst.uddi.client.exceptions.UDDIException;

public class UddiLookupWithoutArsTest {

	private Uri uddiServerUri;

    private final Identifier eanIdentifier = new IdentifierEan("5798009811578");
    private final Identifier dunsIdentifier = new IdentifierDuns("1234567890");

    private final UddiId nonExistingServiceId = new UddiGuidId("uddi:b138dc71-d301-42d1-8c2e-2c3a26fa1111", true);
    private final UddiId orderServiceId = new UddiGuidId("uddi:b138dc71-d301-42d1-8c2e-2c3a26faf56a", true);
    private final UddiId invoiceServiceId = new UddiGuidId("uddi:2e0b402a-7a5e-476b-8686-b33f54fd1f47", true);

    private final UddiId nonExistingProfileUddiId = new UddiGuidId("uddi:88fbd6d5-6a25-4c08-91cc-5344c73c1111", true);
    private final UddiId procurementOrdAdvBilSimProfileUddiId = new UddiGuidId("uddi:88fbd6d5-6a25-4c08-91cc-5344c73c4d69", true);
    private final UddiId nesublProfilesProfile5 = new UddiGuidId("uddi:AEE8B6DE-298F-4cbc-A96D-9AE8AED0AC31", true);

    private final String nonExistingRoleIdentifier = "NonExistingSellerParty";
    private final String sellerPartyRoleIdentifier = "SellerParty";

    private final List<EndpointAddressTypeCode> acceptHttpProtocol = new LinkedList<EndpointAddressTypeCode>();
    private final List<EndpointAddressTypeCode> acceptSmtpProtocol = new LinkedList<EndpointAddressTypeCode>();
    
    @Before
    public void SetupUddi() throws URISyntaxException {
        acceptHttpProtocol.add(EndpointAddressTypeCode.http);
    	acceptSmtpProtocol.add(EndpointAddressTypeCode.email);
        
    	//ConfigurationUtil.SetupConfiguration();
    	ConfigurationHandler.setConfigurationFile(TestConstants.PATH_TEST_RASPCONFIGURATION);
    	
    	UddiConfig config = ConfigurationHandler.getInstance().getUddiConfig();
        uddiServerUri = config.getLookupRegistryFallbackConfig().getRegistries().iterator().next().getEndpoints().iterator().next();
    }

    @Test
	public void LookingUpExistingServiceMustReturnResponseWithValidProperties() throws URISyntaxException {
	    List<UddiId> profileIds = new LinkedList<UddiId>();
	    profileIds.add(procurementOrdAdvBilSimProfileUddiId);
	    
	    LookupParameters lookupParameters = new LookupParameters(eanIdentifier, orderServiceId, profileIds, acceptHttpProtocol);
	    List<UddiLookupResponse> lookupResponses = GetEndpointsWithProfileFromUddi(lookupParameters);
	    Assert.assertEquals("Exactly 1 endpoint expected.", 1, lookupResponses.size());
	
	    UddiLookupResponse response = lookupResponses.get(0);
	    AssertReponsePropertiesAreSetCorrectly(response);
	}
    
    @Test
    public void LookingUpExistingServiceMustReturnCertificateSubjectString() {
        List<UddiId> profileIds = new LinkedList<UddiId>();
        profileIds.add(procurementOrdAdvBilSimProfileUddiId);
        
        LookupParameters lookupParameters = new LookupParameters(eanIdentifier, orderServiceId, profileIds, acceptHttpProtocol);
        List<UddiLookupResponse> lookupResponses = GetEndpointsWithProfileFromUddi(lookupParameters);
        
        Assert.assertTrue(lookupResponses.size() > 0);

        // Notice, CertificateSubject class changes SERIALNUMBER to OID.2.5.4.5, which is why this is tested even though its not what is in UDDI
        String expectedCertificateSubjectString = "OID.2.5.4.5=CVR:26769388-FID:1200406941690 + CN=NemHandel test service (funktionscertifikat), O=IT- og Telestyrelsen // CVR:26769388, C=DK";
        String actualCertificateSubjectString = lookupResponses.get(0).getCertificateSubjectSerialNumber().getSubjectString();
        Assert.assertEquals(expectedCertificateSubjectString, actualCertificateSubjectString);
    }

    @Test
    public void LookingUpNonExistingServiceShouldReturnEmptyResponse() {
        List<UddiId> profileIds = new LinkedList<UddiId>();
        profileIds.add(procurementOrdAdvBilSimProfileUddiId);
        
        LookupParameters lookupParameters = new LookupParameters(eanIdentifier, nonExistingServiceId, profileIds, acceptHttpProtocol);
        
        try {
        	UddiLookupClient lookupClient = new UddiLookupClient(uddiServerUri);
        	
        	lookupClient.lookup(lookupParameters);
		} catch (MainException e) {
			if (e.getCause() instanceof UDDIException)
			{
				return;
			}
		}
		
        Assert.fail("A UDDIException was not thrown");
    }

    @Test
    public void LookingUpExistingServiceProvidingNonExistingRoleMustReturnEmptyResponseList() {
        List<UddiId> profileIds = new LinkedList<UddiId>();
        profileIds.add(nonExistingProfileUddiId);

        LookupParameters lookupParameters = new LookupParameters(eanIdentifier, orderServiceId, profileIds, acceptHttpProtocol);
        List<UddiLookupResponse> lookupResponses = GetEndpointsWithProfileFromUddi(lookupParameters);
        Assert.assertEquals(0, lookupResponses.size());
    }

    @Test
    public void LookingUpServiceProvidingExistingRoleMustReturnResponse() {
        List<UddiId> profileIds = new LinkedList<UddiId>();
        profileIds.add(procurementOrdAdvBilSimProfileUddiId);

        LookupParameters lookupParameters = new LookupParameters(eanIdentifier, orderServiceId, profileIds, acceptHttpProtocol, sellerPartyRoleIdentifier);
        List<UddiLookupResponse> lookupResponses = GetEndpointsWithProfileFromUddi(lookupParameters);
        Assert.assertEquals(1, lookupResponses.size());
    }
    
    @Test
    public void LookingUpExistingServiceWithoutRoleMustReturnEmptyResponseList() {
        List<UddiId> profileIds = new LinkedList<UddiId>();
        profileIds.add(procurementOrdAdvBilSimProfileUddiId);

        LookupParameters lookupParameters = new LookupParameters(eanIdentifier, orderServiceId, profileIds, acceptHttpProtocol, nonExistingRoleIdentifier);
        List<UddiLookupResponse> lookupResponses = GetEndpointsWithProfileFromUddi(lookupParameters);
        Assert.assertEquals(0, lookupResponses.size());
    }

    @Test
    public void LookingUpExistingServiceTwoProfilesBothExistingMustReturnResponse() {
        List<UddiId> profileIds = new LinkedList<UddiId>();
        profileIds.add(procurementOrdAdvBilSimProfileUddiId);
        profileIds.add(nesublProfilesProfile5);

        LookupParameters lookupParameters = new LookupParameters(eanIdentifier, invoiceServiceId, profileIds, acceptHttpProtocol);
        List<UddiLookupResponse> lookupResponses = GetEndpointsWithProfileFromUddi(lookupParameters);
        Assert.assertEquals(1, lookupResponses.size());
    }

    @Test
    public void LookingUpServiceProvidingNoProfilesWhereServiceHasProfileMustReturnResponse() {
    	LookupParameters lookupParameters = new LookupParameters(eanIdentifier, invoiceServiceId, acceptHttpProtocol);
    	List<UddiLookupResponse> lookupResponses = GetEndpointsWithProfileFromUddi(lookupParameters);
        Assert.assertEquals(1, lookupResponses.size());
    }
    
    @Test
    public void LookingUpExistingServiceTwoProfilesOneExistingMustReturnResponse() {
        List<UddiId> profileIds = new LinkedList<UddiId>();
        profileIds.add(nonExistingProfileUddiId);
        profileIds.add(nesublProfilesProfile5);

        LookupParameters lookupParameters = new LookupParameters(eanIdentifier, invoiceServiceId, profileIds, acceptHttpProtocol);
        List<UddiLookupResponse> lookupResponses = GetEndpointsWithProfileFromUddi(lookupParameters);
        Assert.assertEquals(1, lookupResponses.size());
    }

    @Test
    public void LookingUpExistingServiceWithDunsIdentifierShouldReturnResponse() {
        List<UddiId> profileIds = new LinkedList<UddiId>();
        profileIds.add(procurementOrdAdvBilSimProfileUddiId);

        LookupParameters lookupParameters = new LookupParameters(dunsIdentifier, invoiceServiceId, profileIds, acceptHttpProtocol);
        List<UddiLookupResponse> lookupResponses = GetEndpointsWithProfileFromUddi(lookupParameters);
        Assert.assertEquals(1, lookupResponses.size());
    }

    @Test
    public void LookingUpHttpOnlyServiceWithAcceptedProtocolTypeSetToSmtpShouldReturnEmptyResponse() {
        List<UddiId> profileIds = new LinkedList<UddiId>();
        profileIds.add(procurementOrdAdvBilSimProfileUddiId);

        LookupParameters lookupParameters = new LookupParameters(eanIdentifier, invoiceServiceId, profileIds, acceptSmtpProtocol);
        List<UddiLookupResponse> lookupResponses = GetEndpointsWithProfileFromUddi(lookupParameters);
        Assert.assertEquals(0, lookupResponses.size());
    }
    
    @Test
    public void LookingUpAllRegistrationsOnEanNumber() {
         LookupParameters lookupParameters = new LookupParameters(eanIdentifier, acceptHttpProtocol);
         List<UddiLookupResponse> lookupResponses = GetEndpointsWithProfileFromUddi(lookupParameters);
         Assert.assertTrue(lookupResponses.size()>0);
    }
    
    // Helper methods
    
    private void AssertReponsePropertiesAreSetCorrectly(UddiLookupResponse response) throws URISyntaxException {
        Date expectedActivationDate = new GregorianCalendar(2009, 11, 7, 0, 0, 0).getTime();
        Assert.assertEquals(expectedActivationDate, response.getActivationDate());

        String expectedEndpoint = "http://raspservice.nemhandel.gov.dk/RASP/1.2/reciever.svc";
        Assert.assertEquals(expectedEndpoint, response.getEndpointAddress().getAsUri().getAbsoluteUri());

        String expectedIdentifierActual = "5798009811578";
        Assert.assertEquals(expectedIdentifierActual, response.getEndpointIdentifierActual().getAsString());

        Date expectedExpirationDate = new GregorianCalendar(3009, 11, 7, 0, 0, 0).getTime();
        Assert.assertEquals(expectedExpirationDate, response.getExpirationDate());

        Assert.assertEquals(false, response.hasNewerVersion());

        // TODO: This property is not present on any of the services in uddi and has not been tested
        Assert.assertEquals(null, response.getNewerVersionReference());

        for (ProcessRoleDefinition processRoleDefinition : response.getProcesses()) {
            Assert.assertEquals("Procurement-OrdAdv-BilSim-1.0 SellerParty", processRoleDefinition.getName());
            break;
        }

        // TODO: This property is not present on any of the services in uddi and has not been tested
        String contactEmail = "christian.u.pedersen@accenture.com"; 
        Assert.assertEquals(contactEmail, response.getServiceContactEmail().toString());

        // TODO: This property is not present on any of the services in uddi and has not been tested
        Assert.assertEquals(null, response.getTermsOfUseUrl());

        Version expectedVersion = new Version(1, 0, 0);
        Assert.assertEquals(expectedVersion.toString(), response.getVersion().toString());
    }
    
    private List<UddiLookupResponse> GetEndpointsWithProfileFromUddi(LookupParameters lookupParameters) {
        try {
        	UddiLookupClient lookupClient = new UddiLookupClient(uddiServerUri);
        	
			return lookupClient.lookup(lookupParameters);
		} catch (MainException e) {
			return null;
		}
    }
}
