/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis Soegaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */

package dk.gov.oiosi.test.unit.addressing;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.addressing.IdentifierCvr;
import dk.gov.oiosi.addressing.IdentifierEan;
import dk.gov.oiosi.addressing.IdentifierOvt;
import dk.gov.oiosi.addressing.IncorrectBusinessIdentifierException;
import junit.framework.TestCase;

/**
 * test rasp cache.
 */
public class IdentifierTest extends TestCase {

    /**
     * logger.
     */
    private static Log log = LogFactory.getLog(IdentifierTest.class);

    /**
     * provoke IllegalArgumentException for CVR.
     */
    public static final String[] ILLEGAL_EMPTY = new String[] {null, ""};

    /**
     * check IdentifierCvr validator.
     */
    public static final String[] CVR_ILLEGAL_WRONG_FORMAT = new String[] {"DK", "D", "DK1", "DK12", "DK1234567", "DK123456789",
            "1", "1234567", "123456789", "SE12345678", "1 3 5 78", "00TEST00"};

    /**
     * legal CVR.
     */
    public static final String[] CVR_OK = new String[] {"12345678", "DK12345678"};

    /**
     * check IdentifierCvr validator.
     */
    public static final String[] EAN_ILLEGAL_WRONG_FORMAT = new String[] {"1", "12", "123456789012", "12345678901234",
            "1 3 5 7 9 1 3", "1234TEST90123"};

    /**
     * legal CVR.
     */
    public static final String[] EAN_OK = new String[] {"5798009811578"};

    /**
     * test that empty arguments are detected.
     */
    public void testCvrEmpty() {

        for (int i = 0; i < ILLEGAL_EMPTY.length; i++) {
            try {
                new IdentifierCvr(ILLEGAL_EMPTY[i]);
                log.error("Test should fail with illegal argument for : [" + ILLEGAL_EMPTY[i] + "]");
                fail("Test should fail with illegal argument for : [" + ILLEGAL_EMPTY[i] + "]");
            } catch (IllegalArgumentException e) {
                log.info("Failed as expected   : [" + ILLEGAL_EMPTY[i] + "]");
            }
        }
    }

    /**
     * test that wrong format detected.
     */
    public void testCvrWrongFormat() {

        for (int i = 0; i < CVR_ILLEGAL_WRONG_FORMAT.length; i++) {
            try {
                new IdentifierCvr(CVR_ILLEGAL_WRONG_FORMAT[i]);
                log.error("Test should fail with IncorrectBusinessIdentifierException for : [" + CVR_ILLEGAL_WRONG_FORMAT[i] + "]");
                fail("Test should fail with IncorrectBusinessIdentifierException for : [" + CVR_ILLEGAL_WRONG_FORMAT[i] + "]");
            } catch (IncorrectBusinessIdentifierException e) {
                log.info("Failed as expected   : [" + CVR_ILLEGAL_WRONG_FORMAT[i] + "]");
            }
        }
    }

    /**
     * test that legal CVR's are accepted.
     */
    public void testCvrOk() {
        for (int i = 0; i < CVR_OK.length; i++) {
            try {
                new IdentifierCvr(CVR_OK[i]);
                log.info("Accepted as expected : [" + CVR_OK[i] + "]");
            } catch (Throwable e) {
                log.error("Positive Test should no fail for : [" + CVR_OK[i] + "]" + e, e);
                fail("Positive Test should no fail for : [" + CVR_OK[i] + "]" + e);
            }
        }
    }

    /**
     * test that empty arguments are detected.
     */
    public void testEanEmpty() {

        for (int i = 0; i < ILLEGAL_EMPTY.length; i++) {
            try {
                new IdentifierEan(ILLEGAL_EMPTY[i]);
                log.error("Test should fail with illegal argument for : [" + ILLEGAL_EMPTY[i] + "]");
                fail("Test should fail with illegal argument for : [" + ILLEGAL_EMPTY[i] + "]");
            } catch (IllegalArgumentException e) {
                log.info("Failed as expected   : [" + ILLEGAL_EMPTY[i] + "]");
            }
        }
    }

    /**
     * test that wrong format detected.
     */
    public void testEanWrongFormat() {

        for (int i = 0; i < EAN_ILLEGAL_WRONG_FORMAT.length; i++) {
            try {
                new IdentifierEan(EAN_ILLEGAL_WRONG_FORMAT[i]);
                log.error("Test should fail with IncorrectBusinessIdentifierException for : [" + EAN_ILLEGAL_WRONG_FORMAT[i] + "]");
                fail("Test should fail with IncorrectBusinessIdentifierException for : [" + EAN_ILLEGAL_WRONG_FORMAT[i] + "]");
            } catch (IncorrectBusinessIdentifierException e) {
                log.info("Failed as expected   : [" + EAN_ILLEGAL_WRONG_FORMAT[i] + "]");
            }
        }
    }

    /**
     * test that legal CVR's are accepted.
     */
    public void testEanOk() {
        for (int i = 0; i < EAN_OK.length; i++) {
            try {
                new IdentifierEan(EAN_OK[i]);
                log.info("Accepted as expected : [" + EAN_OK[i] + "]");
            } catch (Throwable e) {
                log.error("Positive Test should no fail for : [" + EAN_OK[i] + "]" + e, e);
                fail("Positive Test should no fail for : [" + EAN_OK[i] + "]" + e);
            }
        }

    }


    /**
     * test that empty arguments are detected.
     */
    public void testOvtEmpty() {

        for (int i = 0; i < ILLEGAL_EMPTY.length; i++) {
            try {
                new IdentifierOvt(ILLEGAL_EMPTY[i]);
                log.error("Test should fail with illegal argument for : [" + ILLEGAL_EMPTY[i] + "]");
                fail("Test should fail with illegal argument for : [" + ILLEGAL_EMPTY[i] + "]");
            } catch (IllegalArgumentException e) {
                log.info("Failed as expected   : [" + ILLEGAL_EMPTY[i] + "]");
            }
        }
        for (int i = 0; i < ILLEGAL_EMPTY.length; i++) {
            try {
                new IdentifierOvt(ILLEGAL_EMPTY[i], "12345678", "12345");
                log.error("Test should fail with illegal argument for : [" + ILLEGAL_EMPTY[i] + "]");
                fail("Test should fail with illegal argument for : [" + ILLEGAL_EMPTY[i] + "]");
            } catch (IllegalArgumentException e) {
                log.info("Failed as expected   : [" + ILLEGAL_EMPTY[i] + "]");
            }
        }
        for (int i = 0; i < ILLEGAL_EMPTY.length; i++) {
            try {
                new IdentifierOvt("1234", ILLEGAL_EMPTY[i], "12345");
                log.error("Test should fail with illegal argument for : [" + ILLEGAL_EMPTY[i] + "]");
                fail("Test should fail with illegal argument for : [" + ILLEGAL_EMPTY[i] + "]");
            } catch (IllegalArgumentException e) {
                log.info("Failed as expected   : [" + ILLEGAL_EMPTY[i] + "]");
            }
        }
        for (int i = 0; i < ILLEGAL_EMPTY.length; i++) {
            try {
                new IdentifierOvt("1234", "12345678", ILLEGAL_EMPTY[i]);
                log.error("Test should fail with illegal argument for : [" + ILLEGAL_EMPTY[i] + "]");
                fail("Test should fail with illegal argument for : [" + ILLEGAL_EMPTY[i] + "]");
            } catch (IllegalArgumentException e) {
                log.info("Failed as expected   : [" + ILLEGAL_EMPTY[i] + "]");
            }
        }

    }

    /**
     * check IdentifierCvr validator.
     */
    public static final String[] OVT_ILLEGAL_WRONG_FORMAT = new String[] {"1", "12", "1234123456781234", "123412345678123456",
            "1 3 1234567812345", "12341 3 5 7 12345", "1234123456781 3 5", "1OVT1234567812345" , "123412OVT67812345", "1234123456781OVT5"};

    public static final String[] OVT_ILLEGAL_CONTRY_CODE_WRONG_FORMAT = new String[] {"1", "123", "12345" };

    public static final String[] OVT_ILLEGAL_SERIAL_NUMBER_WRONG_FORMAT = new String[] {"1", "1234", "123456" };

    /**
     * legal CVR.
     */
    public static final String[] OVT_OK = new String[] {"12341234567812345"};

    /**
     * test that wrong format detected.
     */
    public void testOvtWrongFormat() {

        for (int i = 0; i < OVT_ILLEGAL_WRONG_FORMAT.length; i++) {
            try {
                new IdentifierOvt(OVT_ILLEGAL_WRONG_FORMAT[i]);
                log.error("Test should fail with IncorrectBusinessIdentifierException for : [" + OVT_ILLEGAL_WRONG_FORMAT[i] + "]");
                fail("Test should fail with IncorrectBusinessIdentifierException for : [" + OVT_ILLEGAL_WRONG_FORMAT[i] + "]");
            } catch (IncorrectBusinessIdentifierException e) {
                log.info("Failed as expected   : [" + OVT_ILLEGAL_WRONG_FORMAT[i] + "]");
            }
        }

        for (int i = 0; i < OVT_ILLEGAL_CONTRY_CODE_WRONG_FORMAT.length; i++) {
            try {
                new IdentifierOvt(OVT_ILLEGAL_CONTRY_CODE_WRONG_FORMAT[i], "12345678", "12345");
                log.error("Test should fail with IncorrectBusinessIdentifierException for : [" + OVT_ILLEGAL_CONTRY_CODE_WRONG_FORMAT[i] + "]");
                fail("Test should fail with IncorrectBusinessIdentifierException for : [" + OVT_ILLEGAL_CONTRY_CODE_WRONG_FORMAT[i] + "]");
            } catch (IncorrectBusinessIdentifierException e) {
                log.info("Failed as expected   : [" + OVT_ILLEGAL_CONTRY_CODE_WRONG_FORMAT[i] + "]");
            }
        }
        for (int i = 0; i < CVR_ILLEGAL_WRONG_FORMAT.length; i++) {
            try {
                new IdentifierOvt("1234", CVR_ILLEGAL_WRONG_FORMAT[i], "12345");
                log.error("Test should fail with IncorrectBusinessIdentifierException for : [" + CVR_ILLEGAL_WRONG_FORMAT[i] + "]");
                fail("Test should fail with IncorrectBusinessIdentifierException for : [" + CVR_ILLEGAL_WRONG_FORMAT[i] + "]");
            } catch (IncorrectBusinessIdentifierException e) {
                log.info("Failed as expected   : [" + CVR_ILLEGAL_WRONG_FORMAT[i] + "]");
            }
        }
        for (int i = 0; i < OVT_ILLEGAL_SERIAL_NUMBER_WRONG_FORMAT.length; i++) {
            try {
                new IdentifierOvt("1234", "12345678", OVT_ILLEGAL_SERIAL_NUMBER_WRONG_FORMAT[i]);
                log.error("Test should fail with IncorrectBusinessIdentifierException for : [" + OVT_ILLEGAL_SERIAL_NUMBER_WRONG_FORMAT[i] + "]");
                fail("Test should fail with IncorrectBusinessIdentifierException for : [" + OVT_ILLEGAL_SERIAL_NUMBER_WRONG_FORMAT[i] + "]");
            } catch (IncorrectBusinessIdentifierException e) {
                log.info("Failed as expected   : [" + OVT_ILLEGAL_SERIAL_NUMBER_WRONG_FORMAT[i] + "]");
            }
        }

    }

    /**
     * test that legal CVR's are accepted.
     */
    public void testOvtOk() {
        for (int i = 0; i < OVT_OK.length; i++) {
            try {
                IdentifierOvt identifier = new IdentifierOvt(OVT_OK[i]);
                assertNotNull(identifier);
                log.info("Accepted as expected : [" + OVT_OK[i] + "]");
            } catch (Throwable e) {
                log.error("Positive Test should no fail for : [" + OVT_OK[i] + "]" + e, e);
                fail("Positive Test should no fail for : [" + OVT_OK[i] + "]" + e);
            }
        }

        for (int i = 0; i < CVR_OK.length; i++) {
            try {
                IdentifierOvt identifier = new IdentifierOvt("1234", CVR_OK[i], "12345");
                IdentifierOvt identifier2 = new IdentifierOvt("1234", new IdentifierCvr(CVR_OK[i]), "12345");
                assertNotNull(identifier);
                assertNotNull(identifier2);
                log.info("Accepted as expected : [" + CVR_OK[i] + "]");
            } catch (Throwable e) {
                log.error("Positive Test should no fail for : [" + CVR_OK[i] + "]" + e, e);
                fail("Positive Test should no fail for : [" + CVR_OK[i] + "]" + e);
            }
        }
    }


    public void testCvrEquals() {
        IdentifierCvr i1 = new IdentifierCvr("12345678");
        IdentifierCvr i2 = new IdentifierCvr("12345678");
        IdentifierCvr i3 = new IdentifierCvr("87654321");
        String cvr = "";
        for(int i=0; i<8; i++) {
            cvr += new String( "" +(i+1));
        }
        IdentifierCvr i4 = new IdentifierCvr(cvr);

        assertEquals(true, i1.equals(i2));
        assertEquals(false, i1.equals(i3));
        assertEquals(true, i1.equals(i4));
        log.info("CVR Equeals test parsed...");
    }

    public void testEanEquals() {
        IdentifierEan i1 = new IdentifierEan("1234567890123");
        IdentifierEan i2 = new IdentifierEan("1234567890123");
        IdentifierEan i3 = new IdentifierEan("3210987654321");

        String ean = "";
        for(int i=0; i<13; i++) {
            ean += new String( "" +((i+1)%10));
        }
        IdentifierEan i4 = new IdentifierEan(ean);

        assertEquals(true, i1.equals(i2));
        assertEquals(false, i1.equals(i3));
        assertEquals(true, i1.equals(i4));
        log.info("EAN Equeals test parsed...");
    }

    public void testOvtEquals() {
        IdentifierOvt i1 = new IdentifierOvt("12345678901234567");
        IdentifierOvt i2 = new IdentifierOvt("12345678901234567");
        IdentifierOvt i3 = new IdentifierOvt("76543210987654321");
        String ovt = "";
        for(int i=0; i<17; i++) {
            ovt += new String( "" +((i+1)%10));
        }
        IdentifierOvt i4 = new IdentifierOvt(ovt);

        assertEquals(true, i1.equals(i2));
        assertEquals(false, i1.equals(i3));
        assertEquals(true, i1.equals(i4));
        log.info("OVT Equeals test parsed...");
    }

}
