/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis Soegaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */

package dk.gov.oiosi.test.unit.common.cache;

import junit.framework.TestCase;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.common.OutVariable;
import dk.gov.oiosi.common.TimeSpan;
import dk.gov.oiosi.common.cache.TimedCache;

/**
 * test rasp cache.
 */
public class TimedCacheTest extends TestCase {

    /**
     * local log.
     */
    private static Log log = LogFactory.getLog(TimedCacheTest.class);


    /**
     * cache.
     */
    private TimedCache<String, String> cache;

    /**
     * test constructors.
     */
    public void testConstructors() {
        log.info("Test Constructors");
        TimedCache<String, String> c1 = new TimedCache<String, String>(60000);
        TimedCache<String, String> c2 = new TimedCache<String, String>(new TimeSpan().setMinuttes(1));
        assertNotNull(c1);
        assertNotNull(c2);
    }

    /**
     * add/remove element.
     */
    public void testSingleAddRemoveTest() {
        log.info("Single Add Remove Test Started");

        cache = new TimedCache<String, String>(new TimeSpan().setHours(5));
        cache.add("test", "test");
        testExists("test");
        cache.remove("test");
        testDoNotExists("test");
        log.info("Single Add Remove Test Completed");
    }

    /**
     * add/remove several elements.
     */
    public void testMultipleAddRemoveTest() {
        log.info("Multiple Add Remove Test Stated");

        cache = new TimedCache<String, String>(new TimeSpan().setHours(5));
        String[] a = new String[] {"a1", "a2", "a3"};
        String[] b = new String[] {"b1", "b2"};
        String[] c = new String[] {"c1", "c2", "c3", "c4"};

        addStrings(a);
        addStrings(b);
        checkStringsExists(a);
        checkStringsExists(b);

        removeStrings(b);
        checkStringsExists(a);

        addStrings(c);
        checkStringsExists(a);
        checkStringsExists(c);

        log.info("Multiple Add Remove Test Completed");
    }

    /**
     * add element, let cache timeout element, try to get element.
     */
    public void testSingleTimeoutRemovalTest() {
        log.info("Single Timeout Removal Test Stated");

        TimeSpan timeout = new TimeSpan().setSeconds(1);
        cache = new TimedCache<String, String>(timeout);
        cache.add("test", "test");

        log.info("Waiting for cache timeout 2x{1}" + timeout.getSpanMillis());
        try {
            Thread.sleep(timeout.getSpanMillis());
            Thread.sleep(timeout.getSpanMillis());
        } catch (Exception ignore) {
        }
        testDoNotExists("test");

        log.info("Single Timeout Removal Test Completed");
    }

    /**
     * add several elements, let cache timeout element, try to get element.
     */
    public void testMultipleTimeoutRemovalTest() {
        log.info("Multiple Timeout Removal Test Stated");

        TimeSpan timeout = new TimeSpan().setSeconds(4);
        TimeSpan oneQuaterTimeout = new TimeSpan().setMillis(timeout.getSpanMillis() / 4);
        String a = "a";
        String b = "b";
        cache = new TimedCache<String, String>(timeout);
        testAdd(a, a);

        log.info("Waiting 2x{1}" + oneQuaterTimeout.getSpanMillis());
        try {
            Thread.sleep(oneQuaterTimeout.getSpanMillis());
            Thread.sleep(oneQuaterTimeout.getSpanMillis());
        } catch (Exception ignore) {
        }

        testAdd(b, b);
        testExists(a);

        log.info("Waiting {1}" + oneQuaterTimeout.getSpanMillis());
        try {
            Thread.sleep(oneQuaterTimeout.getSpanMillis());
        } catch (Exception ignore) {
        }

        testExists(a);
        testExists(b);

        log.info("Waiting 2x{1}" + oneQuaterTimeout.getSpanMillis());
        try {
            Thread.sleep(oneQuaterTimeout.getSpanMillis());
            Thread.sleep(oneQuaterTimeout.getSpanMillis());
        } catch (Exception ignore) {
        }

        testDoNotExists(a);
        testExists(b);

        log.info("Waiting 2x{1}" + oneQuaterTimeout.getSpanMillis());
        try {
            Thread.sleep(oneQuaterTimeout.getSpanMillis());
            Thread.sleep(oneQuaterTimeout.getSpanMillis());
        } catch (Exception ignore) {
        }

        testDoNotExists(a);
        testDoNotExists(b);

        log.info("Multiple Timeout Removal Test Completed");
    }

    /**
     * add/remove element. test for existence.
     */
    public void testTimeoutRemovalWithDelete() {
        log.info("Multiple Timeout Removal Test Stated");

        TimeSpan timeout = new TimeSpan().setSeconds(2);
        TimeSpan halfTimeout = new TimeSpan().setMillis(timeout.getSpanMillis() / 2);
        String a = "a";
        String b = "b";
        cache = new TimedCache<String, String>(timeout);

        testAdd(a, a);
        testAdd(b, b);

        log.info("Waiting {1}" + halfTimeout.getSpanMillis());
        try {
            Thread.sleep(halfTimeout.getSpanMillis());
        }
        catch (Exception ignore) {
        }

        testRemove(a);
        testExists(b);

        log.info("Waiting 2x{1}" + halfTimeout.getSpanMillis());
        try {
            Thread.sleep(halfTimeout.getSpanMillis());
            Thread.sleep(halfTimeout.getSpanMillis());
        }
        catch (Exception ignore) {
        }

        testDoNotExists(a);
        testDoNotExists(b);

        log.info("Multiple Timeout Removal Test Completed");
    }

    /**
     * Test the set method
     */
    public void SetTest() {
        String a = "a";
        String b = "b";
        String c = "c";
        testSet(a, b);
        testSet(a, c);
        testElement(a, c);
        testRemove(a);
        testDoNotExists(a);
    }

    /**
     * add strings to cache.
     * @param strings strings to add
     */
    private void addStrings(final String[] strings) {
        for (int i = 0; i < strings.length; i++) {
            testAdd(strings[i], strings[i]);
        }
    }

    /**
     * remove string element from cache.
     * @param strings strings to remove
     */
    private void removeStrings(final String[] strings) {
        for (int i = 0; i < strings.length; i++) {
            testRemove(strings[i]);
        }
    }

    /**
     * check that string element exists in cache
     * @param strings strings to check
     */
    private void checkStringsExists(final String[] strings) {
        for (int i = 0; i < strings.length; i++) {
            testExists(strings[i]);
        }
    }

    /**
     * add element at test existence.
     * @param key key
     * @param value value
     */
    private void testAdd(final String key, final String value) {
        cache.add(key, value);
        testExists(key);
        testElement(key, value);
    }

    private void testSet(final String key, final String value) {
        cache.set(key, value);
        testExists(key);
        testElement(key, value);
    }

    /**
     * remove element an test for removal.
     * @param key key
     */
    private void testRemove(final String key) {
        cache.remove(key);
        testDoNotExists(key);
    }

    /**
     * test that element exists in cache.
     * @param key key
     */
    private void testExists(final String key) {
        assertTrue(cache.containsKey(key));
        OutVariable<String> result = new OutVariable<String>();
        boolean hasValue  = cache.tryGetValue(key, result);
        assertTrue(hasValue);
        assertTrue(result.getVariable() != null);
    }

    /**
     * test that content of cache is correct.
     * @param key key
     * @param expected expected value
     */
    private void testElement(final String key, final String expected) {
        OutVariable<String> result = new OutVariable<String>();
        cache.tryGetValue(key, result);
        assertTrue(result.getVariable() != null);
        assertEquals(expected, result.getVariable());
    }

    /**
     * test that element is not in cache.
     * @param key key
     */
    private void testDoNotExists(final String key) {
        assertFalse(cache.containsKey(key));
        OutVariable<String> result = new OutVariable<String>();
        assertFalse(cache.tryGetValue(key, result));
        assertNull(result.getVariable());
    }
}
